<?php
defined('ABSPATH') or die('No direct access');
class CWAdmin
{
	use CWFoundationTrait;

	protected $fields;

	/* ---------------------------------------------- */
	/* Admin
	/* ---------------------------------------------- */
	public function register()
	{
		load_plugin_textdomain(CWSLUG, false, CWSLUG . DIRECTORY_SEPARATOR . 'languages');

		add_action('admin_init', [$this, 'setAllSettings']);
		add_action('admin_menu', [$this, 'registerMenu']);
		add_action('admin_enqueue_scripts', [$this, 'enqueueAssets']);

		add_filter('plugin_row_meta', [$this, 'registerRowMeta'], 10, 2);
		add_filter('plugin_action_links_' . CWNAME, [$this, 'registerActions']);
		add_filter('plugins_api', [$this, 'plugInInfo'], 20, 3);
		add_filter('site_transient_update_plugins', [$this, 'plugInPushUpdate']);

		// Games tables list action hook for setting games status to finished, see CWGamesTable
		add_action('wp_ajax_cw_control_start', [$this, 'ajaxControlStart']);
		// Games tables list action hook for setting games status to finished, see CWGamesTable
		add_action('wp_ajax_cw_control_finish', [$this, 'ajaxControlFinish']);

		// Games tables list action for unpublishing games, see CWGamesTable
		add_action('wp_ajax_cw_action_unpublish', [$this, 'ajaxActionUnpublish']);
		// Games tables list action for publishing games, see CWGamesTable
		add_action('wp_ajax_cw_action_publish_once', [$this, 'ajaxActionPublish']);

		// Games tables list action for editing games, see CWGamesTable
		add_action('wp_ajax_cw_action_edit', [$this, 'ajaxActionEdit']);
		// Games tables list action for deleting games, see CWGamesTable
		add_action('wp_ajax_cw_action_delete', [$this, 'ajaxActionDelete']);

		// Games tables list action for deleting all games, see CWGamesTable
		add_action('wp_ajax_cw_action_truncate', [$this, 'ajaxActionTruncate']);
		// Games tables list action for deleting all games, see CWGamesTable
		add_action('wp_ajax_cw_action_recreate', [$this, 'ajaxActionRecreate']);
	}

	/* ---------------------------------------------- */
	/* ajax Function
	/* ---------------------------------------------- */
	protected function getTableFromRequest()
	{
		$tablename = $_REQUEST['table'];
		switch ($tablename) {
			case 'tournaments':
				$table = new CWTournamentsTable();
				break;
			case 'cashgames':
				$table = new CWCashGamesTable();
				break;
			case 'players':
				$table = new CWPlayersTable();
				break;
			default:
				$table = null;
		}
		return $table;
	}

	public function ajaxControlStart()
	{
		$this->axajControl(CWEnumGameStatus::Running);
		wp_die();
	}

	public function ajaxControlFinish()
	{
		$this->axajControl(CWEnumGameStatus::Finished);
		wp_die();
	}

	protected function axajControl($status)
	{
		if (!($table = $this->getTableFromRequest())) {
			$error_msg = __('Unable to submit this form, please refresh and try again.');
			$res = ['result' => 'table', 'errmsg' => $error_msg];
			wp_send_json_error($res);
			wp_die();
		}
		$table->ajaxControl($status);
		wp_die();
	}

	public function ajaxActionUnpublish()
	{
		$this->axajPublish(CWEnumGamePublish::None);
		wp_die();
	}

	public function ajaxActionPublish()
	{
		$this->axajPublish(CWEnumGamePublish::Once);
		wp_die();
	}

	protected function axajPublish($value)
	{
		if (!($table = $this->getTableFromRequest())) {
			$error_msg = __('Unable to submit this form, please refresh and try again.');
			$res = ['result' => 'table', 'errmsg' => $error_msg];
			wp_send_json_error($res);
			wp_die();
		}
		$table->ajaxPublish($value);
		wp_die();
	}

	public function ajaxActionDelete()
	{
		if (!($table = $this->getTableFromRequest())) {
			$error_msg = __('Unable to send this command, please refresh and try again.');
			$res = ['result' => 'table', 'errmsg' => $error_msg];
			wp_send_json_error($res);
			wp_die();
		}
		$table->ajaxDelete();
		wp_die();
	}

	public function ajaxActionEdit()
	{
		if (!($table = $this->getTableFromRequest())) {
			$error_msg = __('Unable to send this command, please refresh and try again.');
			$res = ['result' => 'table', 'errmsg' => $error_msg];
			wp_send_json_error($res);
			wp_die();
		}
		$table->ajaxEdit();
		wp_die();
	}

	public function ajaxActionTruncate()
	{
		if (!($table = $this->getTableFromRequest())) {
			$error_msg = __('Unable to send this command, please refresh and try again.');
			$res = ['result' => 'table', 'errmsg' => $error_msg];
			wp_send_json_error($res);
			wp_die();
		}
		$table->ajaxTruncate();
		wp_die();
	}

	public function ajaxActionRecreate()
	{
		if (!($table = $this->getTableFromRequest())) {
			$error_msg = __('Unable to send this command, please refresh and try again.');
			$res = ['result' => 'table', 'errmsg' => $error_msg];
			wp_send_json_error($res);
			wp_die();
		}
		$table->ajaxRecreate();
		wp_die();
	}

	public function registerActions($actions)
	{
		// Add custom settings actions
		$actions[] = '<a href=' . admin_url('admin.php?page=cw_options_general') . '>Settings</a>';
		return $actions;
	}

	public function registerRowMeta($links, $file)
	{
		if ($file == CWNAME) {
			$links[] = '<a class="thickbox open-plugin-details-modal" href="' . admin_url('plugin-install.php?tab=plugin-information&plugin=' . CWSLUG . '&TB_iframe=true&width=600&height=550') . '">' . __('View details') . '</a>';
		}
		return $links;
	}

	public function enqueueAssets()
	{
		$page = isset($_GET['page']) ? $_GET['page'] : '';
		if (strlen($page) > 3 && substr($page, 0, 3) == 'cw_') {
			// Add styles to the page
			wp_enqueue_style(CWSLUG, CWURL . 'css/cw-admin.min.css', [], CWPlugIn::VERSION, 'all');
			// Add scripts to the page
			wp_enqueue_script(CWSLUG, CWURL . 'js/cw-admin.min.js', [], CWPlugIn::VERSION, 'all');
		}
	}

	public function registerMenu()
	{
		// User roles
		$user = wp_get_current_user();
		$roles = (array)$user->roles;
		$role = null;
		if (!empty($roles)) {
			if (in_array('administrator', $roles)) {
				$role = 'administrator';
			} elseif (in_array('editor', $roles)) {
				$role = 'editor';
			}
		}

		// Settings
		$active = 0;
		$base_list_menu = null;
		$base_list_renderer = null;
		$titleSettings = __('Settings', CWSLUG);
		add_menu_page('CasinoWare PlugIn', 'CasinoWare', 'manage_options', 'cw_options_general', [$this, 'renderSettingsGeneral'], 'dashicons-admin-settings');
		add_submenu_page('cw_options_general', 'CasinoWare General ' . $titleSettings, __('General', CWSLUG), 'manage_options', 'cw_options_general', [$this, 'renderSettingsGeneral'], null);
		if (0 < $this->generalOptionForKey('tournaments_manager', 0)) {
			add_submenu_page('cw_options_general', 'CasinoWare Design ' . $titleSettings, __('Tournaments', CWSLUG), 'manage_options', 'cw_options_tournaments', [$this, 'renderOptionsTournaments'], null);
			if (0 === $active) {
				$base_list_menu = 'cw_tournaments_list';
				$base_list_renderer = 'renderTournamentsList';
			}
			$active++;
		}
		if (0 < $this->generalOptionForKey('cashgames_manager', 0)) {
			add_submenu_page('cw_options_general', 'CasinoWare Design ' . $titleSettings, __('Cash Games', CWSLUG), 'manage_options', 'cw_options_cashgames', [$this, 'renderOptionsCashGames'], null);
			if (0 === $active) {
				$base_list_menu = 'cw_cashgames_list';
				$base_list_renderer = 'renderCashGamesList';
			}
			$active++;
		}
		if (0 < $this->generalOptionForKey('leaderboard_manager', 0)) {
			add_submenu_page('cw_options_general', 'CasinoWare Design ' . $titleSettings, __('Leaderboards', CWSLUG), 'manage_options', 'cw_options_leaderboards', [$this, 'renderOptionsLeaderboards'], null);
			if (0 === $active) {
				$base_list_menu = 'cw_players_list';
				$base_list_renderer = 'renderPlayersList';
			}
			$active++;
		}
		add_submenu_page('cw_options_general', 'CasinoWare API ' . $titleSettings, 'API', 'manage_options', 'cw_settings_api', [$this, 'renderSettingsApi'], null);

		// Lists tables
		if (0 < $active && !empty($base_list_menu) && !empty($role)) {
			add_menu_page('CasinoWare', 'CasinoWare', $role, $base_list_menu, [$this, $base_list_renderer], 'dashicons-database', 26);
			if (0 < $this->generalOptionForKey('tournaments_manager', 0)) {
				add_submenu_page($base_list_menu, 'Tournaments', __('Tournaments', CWSLUG), $role, 'cw_tournaments_list', [$this, 'renderTournamentsList'], null);
				// add_action("load-$toplevel", [$this, "tounrnamentsScreenOptions"]);
			}
			if (0 < $this->generalOptionForKey('cashgames_manager', 0)) {
				add_submenu_page($base_list_menu, 'Cash Games', __('Cash Games', CWSLUG), $role, 'cw_cashgames_list', [$this, 'renderCashGamesList'], null);
				// add_action("load-$toplevel", [$this, "cashgamesScreenOptions"]);
			}
			if (0 < $this->generalOptionForKey('leaderboard_manager', 0)) {
				add_submenu_page($base_list_menu, 'Players', __('Players', CWSLUG), $role, 'cw_players_list', [$this, 'renderPlayersList'], null);
				// add_action("load-$toplevel", [$this, "playersScreenOptions"]);
			}
		}
	}

	public function playersScreenOptions()
	{
		$args = array(
			'label' => __('Members per page', CWSLUG),
			'default' => 20,
			'option' => 'cw_per_page'
		);
		add_screen_option('per_page', $args);
		// $args = array(
		// 	'label' => __('Members per page', CWSLUG),
		// 	'default' => 2,
		// 	'max' => 3,
		// 	'option' => 'cw_columns'
		// );
		// add_screen_option('layout_columns', $args);
	}

	public function renderTournamentsList()
	{
		$this->render('tournaments');
	}

	public function renderCashGamesList()
	{
		$this->render('cashgames');
	}

	public function renderPlayersList()
	{
		$this->render('players');
	}

	public function render($name)
	{
		$file = CWPATH . "templates/admin-{$name}.php";
		if (!file_exists($file)) {
			echo "ERROR: File missing!";
			return;
		}
		include($file);
	}

	// /* ---------------------------------------------- */
	// /* Rest API
	// /* ---------------------------------------------- */
	// public function initRestApi()
	// {
	// 	register_rest_route(CWSLUG, 'v1/admin', ['methods' => 'POST', 'callback' => [$this, 'handleRequest'], 'permission_callback' => '__return_true']);
	// }

	// public function handleRequest($req)
	// {
	// 	$headers = $req->get_headers();
	// 	$nonce = $headers['x_wp_nonce'][0];
	// 	if (!wp_verify_nonce($nonce, 'wp_test')) {
	// 		return new WP_REST_Response('Error', 422);
	// 	}
	// 	$task = $req->get_param('task');
	// 	$db = $req->get_param('db');
	// 	return new WP_REST_Response('Thanks', 200);
	// }

	/* ---------------------------------------------- */
	/* Update
	/* ---------------------------------------------- */
	public function plugInInfo($res, $action, $args)
	{
		// do nothing if this is not about getting plugin information
		if ('plugin_information' !== $action) {
			return $res;
		}

		// do nothing if it is not our plugin
		if (CWSLUG !== $args->slug) {
			return $res;
		}

		// info.json is the file with the actual plugin information on our server
		$remote = wp_remote_get(CWPlugIn::UPDATE_URI, ['timeout' => 10, 'headers' => ['Accept' => 'application/json']]);

		// do nothing if we don't get the correct response from the server
		if (is_wp_error($remote) || 200 !== wp_remote_retrieve_response_code($remote) || empty(wp_remote_retrieve_body($remote))) {
			return $res;
		}

		$remote = json_decode(wp_remote_retrieve_body($remote));

		$res = new stdClass();
		$res->name = $remote->name;
		$res->slug = $remote->slug;
		$res->author = $remote->author;
		$res->author_profile = $remote->author_profile;
		$res->version = $remote->version;
		$res->tested = $remote->tested;
		$res->requires = $remote->requires;
		$res->requires_php = $remote->requires_php;
		$res->download_link = $remote->download_url;
		$res->trunk = $remote->download_url;
		$res->last_updated = $remote->last_updated;
		$res->sections = array(
			'description' => $remote->sections->description,
			'installation' => $remote->sections->installation,
			'changelog' => $remote->sections->changelog
			// you can add your custom sections (tabs) here
		);
		// in case you want the screenshots tab, use the following HTML format for its content:
		// <ol><li><a href="IMG_URL" target="_blank"><img src="IMG_URL" alt="CAPTION" /></a><p>CAPTION</p></li></ol>
		if (!empty($remote->sections->screenshots)) {
			$res->sections['screenshots'] = $remote->sections->screenshots;
		}

		$res->banners = array(
			'low' => $remote->banners->low,
			'high' => $remote->banners->high
		);

		return $res;
	}

	public function plugInPushUpdate($transient)
	{
		if (empty($transient->checked)) {
			return $transient;
		}

		$remote = wp_remote_get(CWPlugIn::UPDATE_URI, ['timeout' => 20, 'headers' => ['Accept' => 'application/json']]);

		if (is_wp_error($remote) || 200 !== wp_remote_retrieve_response_code($remote) || empty(wp_remote_retrieve_body($remote))) {
			return $transient;
		}

		$remote = json_decode(wp_remote_retrieve_body($remote));

		if ($remote && version_compare(CWPlugIn::VERSION, $remote->version, '<') && version_compare($remote->requires, get_bloginfo('version'), '<') && version_compare($remote->requires_php, PHP_VERSION, '<')) {
			$res = new stdClass();
			$res->slug = $remote->slug;
			$res->plugin = CWNAME;
			$res->new_version = $remote->version;
			$res->tested = $remote->tested;
			$res->package = $remote->download_url;
			$transient->response[$res->plugin] = $res;
			// $transient->checked[$res->plugin] = $remote->version;
		}

		return $transient;
	}

	/* ---------------------------------------------- */
	/* Render
	/* ---------------------------------------------- */
	public function renderSettingsGeneral()
	{
		require_once CWPATH . 'templates/settings-general.php';
	}

	public function renderSettingsApi()
	{
		require_once CWPATH . 'templates/settings-api.php';
	}

	public function renderOptionsTournaments()
	{
		require_once CWPATH . 'templates/settings-tournaments.php';
	}

	public function renderOptionsCashGames()
	{
		require_once CWPATH . 'templates/settings-cashgames.php';
	}

	public function renderOptionsLeaderboards()
	{
		require_once CWPATH . 'templates/settings-leaderboards.php';
	}

	public function registerCustomFields(&$settings, &$sections, &$fields)
	{
		// Register setting
		foreach ($settings as $setting) {
			register_setting($setting['option_group'], $setting['option_name'], $setting['callback']);
		}

		// Add settings section
		foreach ($sections as $section) {
			add_settings_section($section['id'], __($section['title'], CWSLUG), $section['callback'], $section['page']);
		}

		// Store for further use
		$this->fields = $fields;
		// Add settings field
		foreach ($fields as $field) {
			add_settings_field($field['id'],  __($field['title'], CWSLUG),  $field['callback'],  $field['page'],  $field['section'],  $field['args']);
		}
	}

	public function addSectionToFields($section, $options, &$fields)
	{
		foreach ($options as $id => $value) {
			if (gettype($value) == 'array') {
				$type = $value['type'];
				$title = $value['title'];
				$label = isset($value['label']) ? $value['label'] : false;
			} else {
				$type = 'checkbox';
				$title = $value;
				$label = false;
			}
			$args = [
				'type' => $type,
				'option_name' => $section['page'],
				'id' => $id
			];
			if ($label) {
				$args['label'] = $label;
			}
			$fields[] = [
				'id' => $id,
				'title' => $title,
				'callback' => [$this, 'cbOutputField'],
				'page' => $section['page'],
				'section' => $section['id'],
				'args' => $args
			];
		}
	}

	public function setAllSettings()
	{
		$titleSettings = __('Settings', CWSLUG);
		$settings = [
			['option_group' => 'cw_options_general', 'option_name' => 'cw_options_general', 'callback' => [$this, 'cbInputSettingsGeneral']],
			['option_group' => 'cw_options_api', 'option_name' => 'cw_options_api', 'callback' => [$this, 'cbInputSettingsApi']],
			['option_group' => 'cw_options_tournament_schedule', 'option_name' => 'cw_options_tournament_schedule', 'callback' => [$this, 'cbInputOptionsTournamentSchedule']],
			['option_group' => 'cw_options_tournament_list', 'option_name' => 'cw_options_tournament_list', 'callback' => [$this, 'cbInputOptionsTournamentList']],
			['option_group' => 'cw_options_tournament_results', 'option_name' => 'cw_options_tournament_results', 'callback' => [$this, 'cbInputOptionsTournamentResults']],
			['option_group' => 'cw_options_tournament_details', 'option_name' => 'cw_options_tournament_details', 'callback' => [$this, 'cbInputOptionsTournamentDetails']],
			['option_group' => 'cw_options_cashgame_list', 'option_name' => 'cw_options_cashgame_list', 'callback' => [$this, 'cbInputOptionsCashGameList']],
			['option_group' => 'cw_options_cashgame_details', 'option_name' => 'cw_options_cashgame_details', 'callback' => [$this, 'cbInputOptionsCashGameDetails']],
			['option_group' => 'cw_options_leaderboard', 'option_name' => 'cw_options_leaderboard', 'callback' => [$this, 'cbInputOptionsLeaderboard']]
		];
		$sections = [
			['id' => 'cw_options_general', 'title' => __('General', CWSLUG) . ' ' . $titleSettings, 'callback' => [$this, 'cbSection'], 'page' => 'cw_options_general'],
			['id' => 'cw_options_api', 'title' => 'API-' . $titleSettings, 'callback' => [$this, 'cbSection'], 'page' => 'cw_options_api']
		];

		$fields = [
			['id' => 'tournaments_manager', 'title' => 'Tournaments', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'toggle', 'option_name' => 'cw_options_general', 'id' => 'tournaments_manager']],
			['id' => 'cashgames_manager', 'title' => 'Cash Games', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'toggle', 'option_name' => 'cw_options_general', 'id' => 'cashgames_manager']],
			['id' => 'leaderboard_manager', 'title' => 'Leaderboards', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'toggle', 'option_name' => 'cw_options_general', 'id' => 'leaderboard_manager']],
			['id' => 'currency_0', 'title' => 'Main Currency Symbol', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'text', 'option_name' => 'cw_options_general', 'id' => 'currency_0']],
			['id' => 'currency_pos_0', 'title' => 'Main Currency Position', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'currency_pos', 'option_name' => 'cw_options_general', 'id' => 'currency_pos_0']],
			['id' => 'currency_1', 'title' => '2nd Currency Symbol', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'text', 'option_name' => 'cw_options_general', 'id' => 'currency_1']],
			['id' => 'currency_pos_1', 'title' => '2nd Currency Position', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'currency_pos', 'option_name' => 'cw_options_general', 'id' => 'currency_pos_1']],
			['id' => 'currency_2', 'title' => '3rd Currency Symbol', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'text', 'option_name' => 'cw_options_general', 'id' => 'currency_2']],
			['id' => 'currency_pos_2', 'title' => '3rd Currency Position', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'currency_pos', 'option_name' => 'cw_options_general', 'id' => 'currency_pos_2']],
			['id' => 'currency_3', 'title' => '4th Currency Symbol', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'text', 'option_name' => 'cw_options_general', 'id' => 'currency_3']],
			['id' => 'currency_pos_3', 'title' => '4th Currency Position', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'currency_pos', 'option_name' => 'cw_options_general', 'id' => 'currency_pos_3']],
			['id' => 'currency_4', 'title' => '5th Currency Symbol', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'text', 'option_name' => 'cw_options_general', 'id' => 'currency_4']],
			['id' => 'currency_pos_4', 'title' => '5th Currency Position', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_general', 'section' => 'cw_options_general', 'args' => ['type' => 'currency_pos', 'option_name' => 'cw_options_general', 'id' => 'currency_pos_4']],
			['id' => 'sender_uname', 'title' => 'Username', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_api', 'section' => 'cw_options_api', 'args' => ['type' => 'text', 'option_name' => 'cw_options_api', 'id' => 'sender_uname']],
			['id' => 'sender_password', 'title' => 'Password', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_api', 'section' => 'cw_options_api', 'args' => ['type' => 'password', 'option_name' => 'cw_options_api', 'id' => 'sender_password']],
			['id' => 'sender_apikey', 'title' => 'API-Key', 'callback' => [$this, 'cbOutputField'], 'page' => 'cw_options_api', 'section' => 'cw_options_api', 'args' => ['type' => 'text', 'option_name' => 'cw_options_api', 'id' => 'sender_apikey']]
		];

		// $options = [
		// 	'wrapper' => ['title' => "Wrapper", 'type' => 'checkbox', 'label' => esc_html__("Include a wrapper with class 'cw-wrapper'", CWSLUG)],
		// 	'card' => ['title' => esc_html__("Card Background", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show Card", CWSLUG)],
		// 	'card_title' => ['title' => "Card Header", 'type' => 'text'],
		// 	'card_footer' => ['title' => "Card Footer", 'type' => 'text'],
		// 	'table_headers' => ['title' => esc_html__("Table Headers", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show headers", CWSLUG)],
		// 	'grouped' => ['title' => esc_html__("Group by date", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show groups", CWSLUG)],
		// 	'drawer' => ['title' => "Open Drawer at Size", 'type' => 'breakpoints'],
		// 	'details_page_id' => ['title' => "Details Page", 'type' => 'wp_dropdown_pages', 'label' => "Page"]
		// ];
		// $sections[] = $section;
		// $this->addSectionToFields($section, $options, $fields);

		// Tournament Schedule Layout section and fields
		$section = [
			'id' => 'cw_options_tournament_schedule_main',
			'title' => 'Tournament Schedule',
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_tournament_schedule'
		];
		$options = [
			'wrapper' => ['title' => "Wrapper", 'type' => 'checkbox', 'label' => esc_html__("Include a wrapper with class 'cw-wrapper'", CWSLUG)],
			'card' => ['title' => esc_html__("Card Background", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show Card", CWSLUG)],
			'card_title' => ['title' => "Card Header", 'type' => 'text'],
			'card_footer' => ['title' => "Card Footer", 'type' => 'text'],
			'table_headers' => ['title' => esc_html__("Table Headers", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show headers", CWSLUG)],
			'grouped' => ['title' => esc_html__("Group by date", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show groups", CWSLUG)],
			'drawer' => ['title' => "Open Drawer at Size", 'type' => 'breakpoints'],
			'details_page_id' => ['title' => "Details Page", 'type' => 'wp_dropdown_pages', 'label' => "Page"],
			'limit' => ['title' => "Max Number of Rows per Page", 'type' => 'dropdown_limits', 'label' => "Max Games"],
			'empty_list' => ['title' => esc_html__("Empty List", CWSLUG), 'type' => 'text']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Tournament Schedule Columns section and fields
		$section = [
			'id' => 'cw_options_tournament_schedule_columns',
			'title' => esc_html__("Columns", CWSLUG),
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_tournament_schedule'
		];
		$options = [
			'color' => ['title' => "Color", 'type' => 'column_enabled'],
			'start_date_local' => ['title' => "Date", 'type' => 'column_options'],
			'start_time_local' => ['title' => "Time", 'type' => 'column_options'],
			'start_date_time_local' => ['title' => "Date & Time", 'type' => 'column_options'],
			'title' => ['title' => "Title", 'type' => 'column_options'],
			'shortlimitgame' => ['title' => "Short Limit & Game", 'type' => 'column_options'],
			'costspb0' => ['title' => "Costs (Buy-In & Fee)", 'type' => 'column_options'],
			'guaranteed0' => ['title' => "Guaranteed Prize Pool", 'type' => 'column_options'],
			'prizepool0' => ['title' => "Prize Pool", 'type' => 'column_options'],
			'pcol' => ['title' => "Own Column", 'type' => 'column_options'],
			'level_duration' => ['title' => "1st Level Duration", 'type' => 'column_options'],
			'additional_buyins' => ['title' => "Additional Buy-In Options", 'type' => 'column_options'],
			'startchips' => ['title' => "Startstack / Chips", 'type' => 'column_options'],
			'totalens' => ['title' => "Total Entries", 'type' => 'column_options'],
			'countpls' => ['title' => "Number of Players", 'type' => 'column_options'],
			'countwas' => ['title' => "Number of Waiting Players", 'type' => 'column_options'],
			'xlevel' => ['title' => "Late Registration", 'type' => 'column_options'],
			'reg' => ['title' => "Registration Status", 'type' => 'column_options'],
			'status' => ['title' => "Tournament Status", 'type' => 'column_options'],
			'shortdesc' => ['title' => "Short Description", 'type' => 'column_detail'],
			'levels' => ['title' => "Level Structure", 'type' => 'column_detail'],
			'more' => ['title' => "Link to details page", 'type' => 'column_options'],
			'pulldown' => ['title' => "Pulldown Drawer Button", 'type' => 'column_pulldown']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Tournament List Layout section and fields
		$section = [
			'id' => 'cw_options_tournament_list_main',
			'title' => 'Tournament List',
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_tournament_list'
		];
		$options = [
			'wrapper' => ['title' => "Wrapper", 'type' => 'checkbox', 'label' => esc_html__("Include a wrapper with class 'cw-wrapper'", CWSLUG)],
			'card' => ['title' => esc_html__("Card Background", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show Card", CWSLUG)],
			'card_title' => ['title' => "Card Header", 'type' => 'text'],
			'card_footer' => ['title' => "Card Footer", 'type' => 'text'],
			'table_headers' => ['title' => esc_html__("Table Headers", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show headers", CWSLUG)],
			'grouped' => ['title' => esc_html__("Group by date", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show groups", CWSLUG)],
			'drawer' => ['title' => "Open Drawer at Size", 'type' => 'breakpoints'],
			'details_page_id' => ['title' => "Details Page", 'type' => 'wp_dropdown_pages', 'label' => "Page"],
			'limit' => ['title' => "Max Number of Rows per Page", 'type' => 'dropdown_limits', 'label' => "Max Games"],
			'empty_list' => ['title' => esc_html__("Empty List", CWSLUG), 'type' => 'text']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Tournament List Columns section and fields
		$section = [
			'id' => 'cw_options_tournament_list_columns',
			'title' => esc_html__("Columns", CWSLUG),
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_tournament_list'
		];
		$options = [
			'color' => ['title' => "Color", 'type' => 'column_enabled'],
			'start_date_local' => ['title' => "Date", 'type' => 'column_options'],
			'start_time_local' => ['title' => "Time", 'type' => 'column_options'],
			'start_date_time_local' => ['title' => "Date & Time", 'type' => 'column_options'],
			'title' => ['title' => "Title", 'type' => 'column_options'],
			'shortlimitgame' => ['title' => "Short Limit & Game", 'type' => 'column_options'],
			'costspb0' => ['title' => "Costs (Buy-In & Fee)", 'type' => 'column_options'],
			'guaranteed0' => ['title' => "Guaranteed Prize Pool", 'type' => 'column_options'],
			'prizepool0' => ['title' => "Prize Pool", 'type' => 'column_options'],
			'pcol' => ['title' => "Own Column", 'type' => 'column_options'],
			'level_duration' => ['title' => "1st Level Duration", 'type' => 'column_options'],
			'additional_buyins' => ['title' => "Additional Buy-In Options", 'type' => 'column_options'],
			'startchips' => ['title' => "Startstack / Chips", 'type' => 'column_options'],
			'totalens' => ['title' => "Total Entries", 'type' => 'column_options'],
			'countpls' => ['title' => "Number of Players", 'type' => 'column_options'],
			'countwas' => ['title' => "Number of Waiting Players", 'type' => 'column_options'],
			'counttables' => ['title' => "Number of Tables", 'type' => 'column_options'],
			'freeseats' => ['title' => "Available Seats", 'type' => 'column_options'],
			'xlevel' => ['title' => "Late Registration", 'type' => 'column_options'],
			'reg' => ['title' => "Registration Status", 'type' => 'column_options'],
			'status' => ['title' => "Tournament Status", 'type' => 'column_options'],
			'shortdesc' => ['title' => "Short Description", 'type' => 'column_detail'],
			'levels' => ['title' => "Level Structure", 'type' => 'column_detail'],
			'more' => ['title' => "Link to details page", 'type' => 'column_options'],
			'pulldown' => ['title' => "Pulldown Drawer Button", 'type' => 'column_pulldown']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Tournament Results Layout section and fields
		$section = [
			'id' => 'cw_options_tournament_results_main',
			'title' => 'Tournament Results',
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_tournament_results'
		];
		$options = [
			'wrapper' => ['title' => "Wrapper", 'type' => 'checkbox', 'label' => esc_html__("Include a wrapper with class 'cw-wrapper'", CWSLUG)],
			'card' => ['title' => esc_html__("Card Background", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show Card", CWSLUG)],
			'card_title' => ['title' => "Card Header", 'type' => 'text'],
			'card_footer' => ['title' => "Card Footer", 'type' => 'text'],
			'table_headers' => ['title' => esc_html__("Table Headers", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show headers", CWSLUG)],
			'grouped' => ['title' => esc_html__("Group by date", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show groups", CWSLUG)],
			'drawer' => ['title' => "Open Drawer at Size", 'type' => 'breakpoints'],
			'details_page_id' => ['title' => "Details Page", 'type' => 'wp_dropdown_pages', 'label' => "Page"],
			'limit' => ['title' => "Max Number of Rows per Page", 'type' => 'dropdown_limits', 'label' => "Max Games"],
			'empty_list' => ['title' => esc_html__("Empty List", CWSLUG), 'type' => 'text']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Tournament Results Columns section and fields
		$section = [
			'id' => 'cw_options_tournament_results_columns',
			'title' => esc_html__("Columns", CWSLUG),
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_tournament_results'
		];
		$options = [
			'color' => ['title' => "Color", 'type' => 'column_enabled'],
			'start_date_local' => ['title' => "Date", 'type' => 'column_options'],
			'start_time_local' => ['title' => "Time", 'type' => 'column_options'],
			'start_date_time_local' => ['title' => "Date & Time", 'type' => 'column_options'],
			'title' => ['title' => "Title", 'type' => 'column_options'],
			'shortlimitgame' => ['title' => "Short Limit & Game", 'type' => 'column_options'],
			'costspb0' => ['title' => "Costs (Buy-In & Fee)", 'type' => 'column_options'],
			'guaranteed0' => ['title' => "Guaranteed Prize Pool", 'type' => 'column_options'],
			'prizepool0' => ['title' => "Prize Pool", 'type' => 'column_options'],
			'pcol' => ['title' => "Own Column", 'type' => 'column_options'],
			'level_duration' => ['title' => "1st Level Duration", 'type' => 'column_options'],
			'additional_buyins' => ['title' => "Additional Buy-In Options", 'type' => 'column_options'],
			'startchips' => ['title' => "Startstack / Chips", 'type' => 'column_options'],
			'totalens' => ['title' => "Total Entries", 'type' => 'column_options'],
			'countpls' => ['title' => "Number of Players", 'type' => 'column_options'],
			'countwas' => ['title' => "Number of Waiting Players", 'type' => 'column_options'],
			'xlevel' => ['title' => "Late Registration", 'type' => 'column_options'],
			'reg' => ['title' => "Registration Status", 'type' => 'column_options'],
			'status' => ['title' => "Tournament Status", 'type' => 'column_options'],
			'shortdesc' => ['title' => "Short Description", 'type' => 'column_detail'],
			'levels' => ['title' => "Level Structure", 'type' => 'column_detail'],
			'more' => ['title' => "Link to details page", 'type' => 'column_options'],
			'pulldown' => ['title' => "Pulldown Drawer Button", 'type' => 'column_pulldown']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Tournament Details Layout section and fields
		$section = [
			'id' => 'cw_options_tournament_details_main',
			'title' => 'Single Tournament Details',
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_tournament_details'
		];
		$options = [
			'wrapper' => ['title' => "Wrapper", 'type' => 'checkbox', 'label' => esc_html__("Include a wrapper with class 'cw-wrapper'", CWSLUG)],
			'card' => ['title' => esc_html__("Card Background", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show Card", CWSLUG)],
			'card_footer' => ['title' => "Card Footer", 'type' => 'text'],
			'entries' => ['title' => "List of Entries", 'type' => 'checkbox', 'label' => esc_html__("Show Entries", CWSLUG)],
			'tables' => ['title' => "List of Tables", 'type' => 'checkbox', 'label' => esc_html__("Show Tables", CWSLUG)],
			'waits' => ['title' => "Waiting List", 'type' => 'checkbox', 'label' => esc_html__("Show Waiting List", CWSLUG)],
			'levels' => ['title' => "Level Structure", 'type' => 'checkbox', 'label' => esc_html__("Show Levels", CWSLUG)],
			'payouts' => ['title' => "Payouts", 'type' => 'checkbox', 'label' => esc_html__("Show Payouts", CWSLUG)],
			'htmldesc' => ['title' => "Description", 'type' => 'checkbox', 'label' => esc_html__("Show Description", CWSLUG)]
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Tournament Details Fields section and fields
		$section = [
			'id' => 'cw_options_tournament_details_fields',
			'title' => 'Fields',
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_tournament_details'
		];
		$options = [
			'costspb0' => ['title' => "Costs (Buy-In & Fee)", 'type' => 'checkbox', 'label' => esc_html__("Show", CWSLUG)],
			'prizepool0' => ['title' => "Prize Pool", 'type' => 'checkbox', 'label' => esc_html__("Show", CWSLUG)],
			'additional_buyins' => ['title' => "Additional Buy-In Options", 'type' => 'checkbox', 'label' => esc_html__("Show", CWSLUG)]
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Cash Game List Layout section and fields
		$section = [
			'id' => 'cw_options_cashgame_list_main',
			'title' => 'Cash Game List',
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_cashgame_list'
		];
		$options = [
			'wrapper' => ['title' => "Wrapper", 'type' => 'checkbox', 'label' => esc_html__("Include a wrapper with class 'cw-wrapper'", CWSLUG)],
			'card' => ['title' => esc_html__("Card Background", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show Card", CWSLUG)],
			'card_title' => ['title' => "Card Header", 'type' => 'text'],
			'card_footer' => ['title' => "Card Footer", 'type' => 'text'],
			'table_headers' => ['title' => esc_html__("Table Headers", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show headers", CWSLUG)],
			'grouped' => ['title' => "Group by game", 'type' => 'checkbox', 'label' => esc_html__("Show groups", CWSLUG)],
			'drawer' => ['title' => "Open Drawer at Size", 'type' => 'breakpoints'],
			'details_page_id' => ['title' => "Details Page", 'type' => 'wp_dropdown_pages', 'label' => "Page"],
			'limit' => ['title' => "Max Number of Rows per Page", 'type' => 'dropdown_limits', 'label' => "Max Games"],
			'empty_list' => ['title' => esc_html__("Empty List", CWSLUG), 'type' => 'text']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Cash Game List Columns section and fields
		$section = [
			'id' => 'cw_options_cashgame_list_columns',
			'title' => esc_html__("Columns", CWSLUG),
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_cashgame_list'
		];
		$options = [
			'color' => ['title' => "Color", 'type' => 'column_enabled'],
			'title' => ['title' => "Title", 'type' => 'column_options'],
			'shortlimitgame' => ['title' => "Short Limit & Game", 'type' => 'column_options'],
			'minmax' => ['title' => "Min Max Buy-In", 'type' => 'column_options'],
			'pcol' => ['title' => "Own Column", 'type' => 'column_options'],
			'totalens' => ['title' => "Total Entries", 'type' => 'column_options'],
			'countpls' => ['title' => "Number of Players", 'type' => 'column_options'],
			'countwas' => ['title' => "Number of Waiting Players", 'type' => 'column_options'],
			'counttables' => ['title' => "Number of Tables", 'type' => 'column_options'],
			'freeseats' => ['title' => "Available Seats", 'type' => 'column_options'],
			'reg' => ['title' => "Registration Status", 'type' => 'column_options'],
			'status' => ['title' => "Status", 'type' => 'column_options'],
			'shortdesc' => ['title' => "Short Description", 'type' => 'column_detail'],
			'more' => ['title' => "Link to details page", 'type' => 'column_options'],
			'pulldown' => ['title' => "Pulldown Drawer Button", 'type' => 'column_pulldown']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Cash Game Details Layout section and fields
		$section = [
			'id' => 'cw_options_cashgame_details_main',
			'title' => 'Single Cash Game Details',
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_cashgame_details'
		];
		$options = [
			'wrapper' => ['title' => "Wrapper", 'type' => 'checkbox', 'label' => esc_html__("Include a wrapper with class 'cw-wrapper'", CWSLUG)],
			'card' => ['title' => esc_html__("Card Background", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show Card", CWSLUG)],
			'card_footer' => ['title' => "Card Footer", 'type' => 'text'],
			'entries' => ['title' => "List of Entries", 'type' => 'checkbox', 'label' => esc_html__("Show Entries", CWSLUG)],
			'tables' => ['title' => "List of Tables", 'type' => 'checkbox', 'label' => esc_html__("Show Tables", CWSLUG)],
			'waits' => ['title' => "Waiting List", 'type' => 'checkbox', 'label' => esc_html__("Show Waiting List", CWSLUG)]
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Leaderboard Layout section and fields
		$section = [
			'id' => 'cw_options_leaderboard_main',
			'title' => 'Leaderboard',
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_leaderboard'
		];
		$options = [
			'wrapper' => ['title' => "Wrapper", 'type' => 'checkbox', 'label' => esc_html__("Include a wrapper with class 'cw-wrapper'", CWSLUG)],
			'card' => ['title' => esc_html__("Card Background", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show Card", CWSLUG)],
			'card_title' => ['title' => "Card Header", 'type' => 'text'],
			'card_footer' => ['title' => "Card Footer", 'type' => 'text'],
			'table_headers' => ['title' => esc_html__("Table Headers", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show headers", CWSLUG)],
			'drawer' => ['title' => "Open Drawer at Size", 'type' => 'breakpoints'],
			'empty_list' => ['title' => esc_html__("Empty List", CWSLUG), 'type' => 'text'],
			'search_bar' => ['title' => esc_html__("Search", CWSLUG), 'type' => 'checkbox', 'label' => esc_html__("Show searchbar", CWSLUG)],
			'search_placeholder' => ['title' => esc_html__("Search placeholder", CWSLUG), 'type' => 'text']
			// 'details_page_id' => ['title' => "Details Page", 'type' => 'wp_dropdown_pages', 'label' => "Page"]
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);

		// Leaderboard Columns section and fields
		$section = [
			'id' => 'cw_options_leaderboard_columns',
			'title' => esc_html__("Columns", CWSLUG),
			'callback' => [$this, 'cbSection'],
			'page' => 'cw_options_leaderboard'
		];
		$options = [
			'rank' => ['title' => "Rank", 'type' => 'column_options'],
			'ensign' => ['title' => "Nation Flag", 'type' => 'column_options'],
			'member' => ['title' => "Address/Member", 'type' => 'column_options'],
			'dname' => ['title' => "Name", 'type' => 'column_options'],
			'mdno' => ['title' => "Media No.", 'type' => 'column_options'],
			'ptime' => ['title' => "Playtime", 'type' => 'column_options'],
			'curcount' => ['title' => "Number of Games", 'type' => 'column_options'],
			'curvalue' => ['title' => "Value/Points", 'type' => 'column_options']
		];
		$sections[] = $section;
		$this->addSectionToFields($section, $options, $fields);


		$this->registerCustomFields($settings, $sections, $fields);
	}

	public function cbOptionGroup($input)
	{
		return $input;
	}

	public function cbSection($args)
	{
		switch ($args['id']) {
			case 'cw_options_tournament_schedule_main':
				$s = "This section applies to the tournament schedule shortcode \"<strong>[cw-tournament-schedule]</strong>\" and sets global parameters for all tournament schedules.<br>Schedules only show <strong>future tournaments</strong>. Expired tournaments are no longer displayed here. <a class=\"cw-tab-ref\" href=\"#tab-2\">Lists</a> show all published tournaments.<br>Some of these settings can be changed via the shortcode for the individual page.
				<ul>
				<li>• lcid: Location ID, shows all tournaments of the set location number (e.g. [cw-tournament-schedule lcid=1])</li>
				<li>• evid: Event ID,  shows all tournaments of the set event number (e.g. [cw-tournament-schedule evid=1])</li>
				<li>• type: Game Type, 0 - Sit&Go, 1 - Ring, 2 - Dated (e.g. [cw-tournament-schedule type=0] shows only Sit&Go tournaments)</li>
				<li>• hide: Hide List, 0 - off, 1 - on (e.g. [cw-tournament-schedule hide=1] hides the entire list when no games are scheduled)</li>
				<li>• card: Card Background, 0 - off, 1 - on (e.g. [cw-tournament-schedule card=0] turns the card background off)</li>
				<li>• title: Card Title, (e.g. [cw-tournament-schedule type=0 title=\"Sit&Go Tables\"])</li>
				</ul>
				";
				break;
			case 'cw_options_tournament_list_main':
				$s = "This section applies to the tournament list shortcode \"<strong>[cw-tournament-list]</strong>\" and sets global parameters for all tournament lists.<br>Some of these settings can be changed via the shortcode for the individual page.
					<ul>
					<li>• lcid: Location ID, shows all tournaments of the set location number (e.g. [cw-tournament-list lcid=1])</li>
					<li>• evid: Event ID,  shows all tournaments of the set event number (e.g. [cw-tournament-list evid=1])</li>
					<li>• type: Game Type, 0 - Sit&Go, 1 - Ring, 2 - Dated (e.g. [cw-tournament-list type=0] shows only Sit&Go tournaments)</li>
					<li>• hide: Hide List, 0 - off, 1 - on (e.g. [cw-tournament-list hide=1] hides the entire list when no games are listed)</li>
					<li>• card: Card Background, 0 - off, 1 - on (e.g. [cw-tournament-list card=0] turns the card background off)</li>
					<li>• title: Card Title, (e.g. [cw-tournament-list type=0 title=\"Sit&Go Information\"])</li>
					</ul>
					";
				break;
			case 'cw_options_tournament_results_main':
				$s = "This section applies to the tournament results shortcode \"<strong>[cw-tournament-results]</strong>\" and sets global parameters for all tournament results.<br>Some of these settings can be changed via the shortcode for the individual page.
						<ul>
						<li>• lcid: Location ID, shows all tournaments of the set location number (e.g. [cw-tournament-results lcid=1])</li>
						<li>• evid: Event ID,  shows all tournaments of the set event number (e.g. [cw-tournament-results evid=1])</li>
						<li>• type: Game Type, 0 - Sit&Go, 1 - Ring, 2 - Dated (e.g. [cw-tournament-results type=0] shows only Sit&Go tournaments)</li>
						<li>• hide: Hide List, 0 - off, 1 - on (e.g. [cw-tournament-results hide=1] hides the entire list when no games are listed)</li>
						<li>• card: Card Background, 0 - off, 1 - on (e.g. [cw-tournament-results card=0] turns the card background off)</li>
						<li>• title: Card Title, (e.g. [cw-tournament-results type=0 title=\"Sit&Go Results\"])</li>
						</ul>
						";
				break;
			case 'cw_options_tournament_details_main':
				$s = "This section applies to the single tournament details shortcode \"<strong>[cw-tournament]</strong>\" and sets global parameters for all tournaments. Depending on the status of the tournament, different information is displayed, e.g. the tournament clock<br>Some of these settings can be changed via the shortcode for the individual page.
							<ul>
							<li>• card: Card Background, 0 - off, 1 - on (e.g. [cw-tournament card=0] turns the card background off)</li>
							<li>• entries: Entries' List, 0 - off, 1 - on (e.g. [cw-tournament entries=1] shows the list of entries)</li>
							<li>• tables: Tables' List, 0 - off, 1 - on (e.g. [cw-tournament tables=1] shows the list of tables)</li>
							<li>• waits: Waiting List, 0 - off, 1 - on (e.g. [cw-tournament waits=1] shows the list of waiting players)</li>
								</ul>
							";
				break;
			case 'cw_options_tournament_schedule_columns':
			case 'cw_options_tournament_list_columns':
			case 'cw_options_tournament_results_columns':
			case 'cw_options_cashgame_list_columns':
				$s = "Check or uncheck the columns you want to show in the tournament listing.";
				break;
			case 'cw_options_cashgame_list_main':
				$s = "This section applies to the cash game list shortcode \"<strong>[cw-cashgame-list]</strong>\" and sets global parameters for all cash game lists.<br>Some of these settings can be changed via the shortcode for the individual page.
						<ul>
						<li>• lcid: Location ID, shows all cash games of the set location number (e.g. [cw-cashgame-list lcid=1])</li>
						<li>• evid: Event ID,  shows all cash games of the set event number (e.g. [cw-cashgame-list evid=1])</li>
						<li>• hide: Hide List, 0 - off, 1 - on (e.g. [cw-cashgame-list hide=1] hides the entire list when no games are listed)</li>
						<li>• card: Card Background, 0 - off, 1 - on (e.g. [cw-cashgame-list card=0] turns the card background off)</li>
						<li>• title: Card Title, (e.g. [cw-cashgame-list type=0 title=\"Running Cash Tables\"])</li>
						</ul>
						";
				break;
			case 'cw_options_leaderboard_main':
				$s = __("This section applies to the leaderboard shortcode \"<strong>[cw-leaderboard]</strong>\" and sets global parameters for all leaderboard.<br>Some of these settings can be changed via the shortcode for the individual page.", CWSLUG);
				$s .= "<ul>";
				$s .= "<li>• " . __("lcid: Location ID, shows all players of the set location number (e.g. [cw-leaderboard lcid=1])", CWSLUG) . "</li>";
				$s .= "<li>• " . __("type: Type of the values, 'stat' - session data, 'earn' - total earnings, 'bal' - balances (e.g. [cw-leaderboard type=stat] show a list of session data)", CWSLUG) . "</li>";
				$s .= "<li>• " . __("cur_id: Current Currency ID, selects the currency, allowed values from 0 to 4 for the different accounts (e.g. [cw-leaderboard cur_id=2])", CWSLUG) . "</li>";
				$s .= "<li>• " . __("cur_title: Column Header for the selected cur_id and type (e.g. [cw-leaderboard cur_title=\"Points\"])", CWSLUG) . "</li>";
				$s .= "<li>• " . __("col_ensign: Enables or hides Nation Flag column, 0 - hide, 1 - show (e.g. [cw-leaderboard col_ensign=0])", CWSLUG) . "</li>";
				$s .= "<li>• " . __("col_ptime: Enables or hides the Playtime column, 0 - hide, 1 - show (e.g. [cw-leaderboard col_ptime=0])", CWSLUG) . "</li>";
				$s .= "<li>• " . __("cur_count: Column Header for the number of games (e.g. [cw-leaderboard cur_count=\"Games\"])", CWSLUG) . "</li>";
				$s .= "<li>• " . __("col_cur_count: Enables or hides Number Of Games column, 0 - hide, 1 - show (e.g. [cw-leaderboard col_cur_count=1])", CWSLUG) . "</li>";
				$s .= "<li>• " . __("search_bar: Shows a serach field at the top of list (e.g. [cw-leaderboard search_bar=1])", CWSLUG) . "</li>";
				$s .= "</ul>";
				break;
			case 'cw_options_api':
				$s = esc_html__("The API section allows CasinoWare to transmit data. Please enter a secure password and username. These values must also be entered into CasinoWare, with which CasinoWare authorizes itself.", CWSLUG);
				break;
			default:
				$s = "";
		}
		echo $s . PHP_EOL;
	}

	public function cbInputSettingsGeneral($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_general');
	}

	public function cbInputSettingsApi($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_api');
	}

	public function cbInputOptionsTournamentSchedule($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_tournament_schedule');
	}

	public function cbInputOptionsTournamentList($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_tournament_list');
	}

	public function cbInputOptionsTournamentResults($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_tournament_results');
	}

	public function cbInputOptionsTournamentDetails($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_tournament_details');
	}

	public function cbInputOptionsCashGameList($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_cashgame_list');
	}

	public function cbInputOptionsCashGameDetails($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_cashgame_details');
	}

	public function cbInputOptionsLeaderboard($input)
	{
		return $this->sanitizeInputFields($input, 'cw_options_leaderboard');
	}

	public function sanitizeInputFields($input, $option_name)
	{
		$output = [];
		foreach ($this->fields as $field) {
			if ($field['page'] == $option_name) {
				$id = $field['id'];
				$type = $field['args']['type'];
				switch ($type) {
					case 'toggle':
					case 'checkbox':
						$output[$id] = (isset($input[$id]) && $input[$id]) ? 1 : 0;
						break;
					case 'column_show':
						$opts = [];
						$opts['enabled'] = (isset($input[$id]['enabled']) && $input[$id]['enabled']) ? 1 : 0;
						$opts['detail'] = (isset($input[$id]['detail']) && $input[$id]['detail']) ? 1 : 0;
						$output[$id] = $opts;
						break;
					case 'column_enabled':
						$opts = [];
						$opts['enabled'] = (isset($input[$id]['enabled']) && $input[$id]['enabled']) ? 1 : 0;
						$output[$id] = $opts;
						break;
					case 'column_detail':
						$opts = [];
						$opts['detail'] = (isset($input[$id]['detail']) && $input[$id]['detail']) ? 1 : 0;
						$output[$id] = $opts;
						break;
					case 'column_pulldown':
						$opts = [];
						$opts['enabled'] = (isset($input[$id]['enabled']) && $input[$id]['enabled']) ? 1 : 0;
						$opts['accordion'] = (isset($input[$id]['accordion']) && $input[$id]['accordion']) ? 1 : 0;
						$opts['width'] = (isset($input[$id]['width'])) ? intval($input[$id]['width']) : 0;
						$output[$id] = $opts;
						break;
					case 'column_options':
						$opts = [];
						$opts['enabled'] = (isset($input[$id]['enabled']) && $input[$id]['enabled']) ? 1 : 0;
						$opts['detail'] = (isset($input[$id]['detail']) && $input[$id]['detail']) ? 1 : 0;
						$opts['title'] = (isset($input[$id]['title'])) ? $input[$id]['title'] : "";
						$opts['width'] = (isset($input[$id]['width'])) ? intval($input[$id]['width']) : 0;
						$opts['align'] = (isset($input[$id]['align'])) ? $input[$id]['align'] : "";
						$opts['collapse'] = (isset($input[$id]['collapse'])) ? $input[$id]['collapse'] : "";
						$output[$id] = $opts;
						break;
					case 'detail_options':
						$opts = [];
						$opts['detail'] = (isset($input[$id]['detail']) && $input[$id]['detail']) ? 1 : 0;
						$opts['title'] = (isset($input[$id]['title'])) ? $input[$id]['title'] : "";
						$output[$id] = $opts;
						break;
					case 'detail_option':
						$opts = [];
						$opts['detail'] = (isset($input[$id]['detail']) && $input[$id]['detail']) ? 1 : 0;
						$output[$id] = $opts;
						break;
					case 'wp_dropdown_pages':
					case 'dropdown_limits':
						$output[$id] = isset($input[$id]) ? (int)$input[$id] : 0;
						break;
					default:
						$output[$id] = (isset($input[$id])) ? $input[$id] : "";
				}
			}
		}
		return $output;
	}

	public function cbOutputField($args)
	{
		switch ($args['type']) {
			case 'text':
				$this->renderTextField($args);
				break;
			case 'password':
				$this->renderPasswordField($args);
				break;
			case 'toggle':
				$this->renderToggleField($args);
				break;
			case 'checkbox':
				$this->renderCheckBoxField($args);
				break;
			case 'column_options':
				$this->renderColumnOptions($args);
				break;
			case 'column_show':
				$this->renderColumnOptions($args, ['enabled', 'detail']);
				break;
			case 'column_enabled':
				$this->renderColumnOptions($args, ['enabled']);
				break;
			case 'column_detail':
				$this->renderColumnOptions($args, ['detail']);
				break;
			case 'column_pulldown':
				$this->renderColumnOptions($args, ['enabled', 'width', 'accordion']);
				break;
			case 'breakpoints':
				$this->renderBreakpoints($args);
				break;
			case 'currency_pos':
				$this->renderCurrencyPosition($args);
				break;
			case 'wp_dropdown_pages':
				$this->renderWPDropdownPages($args);
				break;
			case 'dropdown_limits':
				$this->renderDropdownLimits($args);
				break;
		}
	}

	public function renderCurrencyPosition(&$args)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$name = $args['id'];
		$pos = isset($options[$name]) ? $options[$name] : 0;
		$poses = [0 => "None", 1 => "Before Value", 2 => "After Value"];
		echo "<div>";
		echo "<select name=\"{$option_name}[{$name}]\" id=\"{$name}\">";
		foreach ($poses as $value => $title) {
			$sel = ($pos == $value) ? " selected=\"selected\"" : "";
			$title = __($title, CWSLUG);
			echo "<option value=\"{$value}\"{$sel}>{$title}</option>";
		}
		echo "</select>";
		// echo "<p class=\"description\">Aligns the content of the column cell.</p>";
		echo "</div>";
	}

	public function renderBreakpoints(&$args)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$name = $args['id'];
		$drawer = isset($options[$name]) ? $options[$name] : 0;
		$breakpoints = ['' => "No", 'sm' => "Small (576px)", 'md' => "Medium (768px)", 'lg' => "Large (960px)", 'xl' => "Extra Large (1140px)", 'xxl' => "X-Extra Large (1400px)"];
		echo "<div>";
		echo "<select name=\"{$option_name}[{$name}]\" id=\"{$name}\">";
		foreach ($breakpoints as $value => $title) {
			$sel = ($drawer == $value) ? " selected=\"selected\"" : "";
			echo "<option value=\"{$value}\"{$sel}>{$title}</option>";
		}
		echo "</select>";
		// echo "<p class=\"description\">Aligns the content of the column cell.</p>";
		echo "</div>";
	}

	public function renderWPDropdownPages(&$args)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$name = $args['id'];
		$value = $this->valueForKey($options, $name, '');
		$select_args = ['name' => "{$option_name}[{$name}]", 'id' => $name, 'show_option_none' => "- No Selection -", 'selected' => $value];
		wp_dropdown_pages($select_args);
	}

	public function renderDropdownLimits(&$args)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$name = $args['id'];
		$value = $this->valueForKey($options, $name, 0);
		$limits = [0 => '&infin;', 5 => '5', 10 => '10', 15 => '15', 20 => '20', 25 => '25', 30 => '30', 40 => '40', 50 => '50', 75 => '75', 100 => '100'];
		echo "<select name=\"{$option_name}[{$name}]\" id=\"{$name}\">";
		foreach ($limits as $limit => $title) {
			$sel = ($limit == $value) ? " selected=\"selected\"" : "";
			echo "<option value=\"{$limit}\"{$sel}>{$title}</option>";
		}
		echo "</select>";
	}
	public function renderToggleField(&$args)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$name = $args['id'];
		$checkbox = $this->valueForKey($options, $name, false);
		$checked = $checkbox ? " checked" : "";
		echo "<div class=\"cw-toggle\"><input type=\"checkbox\" id=\"{$name}\" name=\"{$option_name}[{$name}]\" value=\"1\"{$checked}/><label for=\"{$name}\"><div></div></label></div>";
	}

	public function renderCheckBoxField(&$args)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$name = $args['id'];
		$label = isset($args['label']) ? $args['label'] : esc_html__("Enable", CWSLUG);
		$checkbox = isset($options[$name]) ? $options[$name] : false;
		$checked = $checkbox ? " checked" : "";
		echo "<div><label for=\"{$name}_flag\"><input type=\"checkbox\" id=\"{$name}_flag\" name=\"{$option_name}[{$name}]\" value=\"1\"{$checked}/>{$label}</label></div>" . PHP_EOL;
	}

	public function renderTextField(&$args)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$placeholder = isset($args['placeholder']) ? " placeholder=\"{$args['placeholder']}\"" : "";
		$name = $args['id'];
		$value = isset($options[$name]) ? esc_attr($options[$name]) : "";
		echo "<input type=\"text\" class=\"regular-text\" id=\"{$name}\" name=\"{$option_name}[{$name}]\" value=\"{$value}\"{$placeholder}/>";
	}

	public function renderPasswordField(&$args)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$placeholder = isset($args['placeholder']) ? " placeholder=\"{$args['placeholder']}\"" : "";
		$name = $args['id'];
		$value = isset($options[$name]) ? esc_attr($options[$name]) : "";
		echo "<input type=\"password\" class=\"regular-text\" id=\"{$name}\" name=\"{$option_name}[{$name}]\" value=\"{$value}\"{$placeholder}/>";
	}

	public function renderColumnOptions(&$args, $fields = null)
	{
		$option_name = $args['option_name'];
		$options = get_option($option_name);
		$name = $args['id'];
		if (isset($options[$name])) {
			$column = $options[$name];
			$enabled = (isset($column['enabled']) ? $column['enabled'] : false) ? " checked" : "";
			$detail = (isset($column['detail']) ? $column['detail'] : false) ? " checked" : "";
			$title = isset($column['title']) ? $column['title'] : "";
			$width = isset($column['width']) ? $column['width'] : 0;
			$align = isset($column['align']) ? $column['align'] : "center";
			$collapse = isset($column['collapse']) ? $column['collapse'] : 0;
			$accordion = (isset($column['accordion']) ? $column['accordion'] : false) ? " checked" : "";
		} else {
			$enabled = "";
			$detail = "";
			$title = "";
			$width = 0;
			$align = "center";
			$collapse = 0;
			$accordion = 0;
		}
		echo "<fieldset>";

		$b_enabled = $fields === null ? true : in_array('enabled', $fields);
		$b_detail = $fields === null ? true : in_array('detail', $fields);
		if ($b_enabled || $b_detail) {
			echo "<div>";
			if ($b_enabled) {
				echo "<label for=\"{$name}_enabled_flag\"><input type=\"checkbox\" id=\"{$name}_enabled_flag\" name=\"{$option_name}[{$name}][enabled]\" value=\"1\"{$enabled}/>" . __('Show as column', CWSLUG) . "</label>&nbsp;&nbsp;&nbsp;&nbsp;";
			}
			if ($b_detail) {
				echo "<label for=\"{$name}_detail_flag\"><input type=\"checkbox\" id=\"{$name}_detail_flag\" name=\"{$option_name}[{$name}][detail]\" value=\"1\"{$detail}/>" . __('Show in drawer', CWSLUG) . "</label>";
			}
			echo "</div>" . PHP_EOL;
		}

		echo "<table class=\"cw-admin-table\"><tbody>" . PHP_EOL;

		if ($fields === null || in_array('title', $fields)) {
			echo "<tr>";
			echo "<td><label>" . __('Name') . "</label>&nbsp;&nbsp;</td><td><input type=\"text\" class=\"regular-text\" id=\"{$name}_title\" name=\"{$option_name}[{$name}][title]\" value=\"{$title}\"/></td>";
			echo "</tr>";
			// echo "<p class=\"description\">The column name should be as short as possible. A colon for the details is automatically added.</p><br></div>";
		}
		if ($fields === null || in_array('width', $fields)) {
			echo "<tr>";
			echo "<td><label>" . __('Width') . "</label>&nbsp;&nbsp;</td><td><input type=\"number\" class=\"small-text\" id=\"{$name}_width\" name=\"{$option_name}[{$name}][width]\" value=\"{$width}\"/></td>";
			echo "</tr>";
			// echo "<p class=\"description\">The column name should be as short as possible. A colon for the details is automatically added.</p><br></div>";
		}

		if ($fields === null || in_array('align', $fields)) {
			$aligns = ['start' => __('Left'), 'center' => __('Center'), 'end' => __('Right')];
			echo "<tr>";
			echo "<td><label>" . __('Alignment') . "</label>&nbsp;&nbsp;</td><td><select name=\"{$option_name}[{$name}][align]\" id=\"{$name}_align\">";
			foreach ($aligns as $value => $title) {
				$sel = ($align == $value) ? " selected=\"selected\"" : "";
				echo "<option value=\"{$value}\"{$sel}>{$title}</option>";
			}
			echo "</select></td>";
			// echo "<p class=\"description\">Aligns the content of the column cell.</p>";
			echo "</tr>";
		}

		if ($fields === null || in_array('collapse', $fields)) {
			$breakpoints = ['' => "No", 'sm' => "Small (576px)", 'md' => "Medium (768px)", 'lg' => "Large (960px)", 'xl' => "Extra Large (1140px)", 'xxl' => "X-Extra Large (1400px)"];
			echo "<tr>";
			echo "<td><label>" . __('Collapse') . "</label>&nbsp;&nbsp;</td><td><select name=\"{$option_name}[{$name}][collapse]\" id=\"{$name}_collapse\">";
			foreach ($breakpoints as $value => $title) {
				$sel = ($collapse == $value) ? " selected=\"selected\"" : "";
				echo "<option value=\"{$value}\"{$sel}>{$title}</option>";
			}
			echo "</select></td>";
			// echo "<p class=\"description\">Aligns the content of the column cell.</p>";
			echo "</tr>";
		}
		if ($fields && in_array('accordion', $fields)) {
			echo "<tr>";
			echo "<td><label for=\"{$name}_accordion_flag\"><input type=\"checkbox\" id=\"{$name}_accordion_flag\" name=\"{$option_name}[{$name}][accordion]\" value=\"1\"{$accordion}/>" . esc_html__("Accordion", CWSLUG) . "</label></td><td><span class=\"description\">&nbsp;&nbsp;(" . esc_html__("Opens only one drawer, closes others", CWSLUG) . ")</span></td>";
			echo "</tr>";
		}
		echo "</tbody></table>" . PHP_EOL;
	}
}
