<?php
defined('ABSPATH') or die('No direct access');
class CWBaseGame extends CWBase
{
	public $params = null;
	public $entries = null;
	public $tables = null;
	public $waits = null;
	public $levels = null;
	public $payouts = null;

	protected static $s_statusnames = null;
	protected static $s_statusimagenames = null;
	protected static $s_pubsnames = null;

	/* ---------------------------------------------- */
	/* Override
	/* ---------------------------------------------- */
	public function setData($data)
	{
		switch (gettype($data)) {
			case 'string':
				$data = json_decode($data, true, 10);
				break;
			case 'integer':
				$data =  $this->loadData((int)$data);
				break;
		}
		if ('array' !== gettype($data))
			return;
		if ($ID = $this->valueForKey($data, 'ID')) {
			$this->ID = $ID;
		}
		$this->LCID = $this->valueForKey($data, 'LCID');
		$this->modified_utc = $this->valueForKey($data, 'modified_utc');
		$vars = ['entries', 'tables', 'waits', 'levels', 'payouts'];
		foreach ($vars as $var) {
			if (isset($data[$var])) {
				$this->$var = json_decode($data[$var], true, 10);
			}
			unset($data[$var]);
		}
		if (isset($data['params'])) {
			$this->params = json_decode($data['params'], true, 10);
		} else {
			$this->params = json_decode($data, true, 10);
		}
	}

	/* ---------------------------------------------- */
	/* Static Functions
	/* ---------------------------------------------- */
	static public function listOfStatusNames()
	{
		if (null === self::$s_statusnames) {
			$list = [
				CWEnumGameStatus::Initiate => "Initiating",
				CWEnumGameStatus::Register => "Registering",
				CWEnumGameStatus::Seating => "Seating",
				CWEnumGameStatus::Running => "Running",
				CWEnumGameStatus::Finished => "Finished",
				CWEnumGameStatus::PauseClock => "Clock stopped",
				CWEnumGameStatus::PauseSeating => "Pause for Seating",
				CWEnumGameStatus::TimeOut => "Time-Out",
				CWEnumGameStatus::Interest => "Interest"
			];
			self::$s_statusnames = $list;
		}
		return self::$s_statusnames;
	}

	protected function listOfStatusImageNames()
	{
		if (null === self::$s_statusimagenames) {
			$list = [
				CWEnumGameStatus::Initiate => "initiate",
				CWEnumGameStatus::Register => "register",
				CWEnumGameStatus::Seating => "seating",
				CWEnumGameStatus::Running => "running",
				CWEnumGameStatus::Finished => "finished",
				CWEnumGameStatus::PauseClock => "pauseclock",
				CWEnumGameStatus::PauseSeating => "pauseseating",
				CWEnumGameStatus::TimeOut => "timeout"
			];
			self::$s_statusimagenames = $list;
		}
		return self::$s_statusimagenames;
	}

	public function listOfPublications()
	{
		if (null === self::$s_pubsnames) {
			$list = [
				0 => "Not published",
				1 => "Once",
				2 => "On registration",
				3 => "Always"
			];
			self::$s_pubsnames = $list;
		}
		return self::$s_pubsnames;
	}

	/* ---------------------------------------------- */
	/* Status
	/* ---------------------------------------------- */
	public function status($base = false)
	{
		if (!isset($this->params['status'])) {
			return 0;
		}
		$status = intval($this->params['status']);
		if ($base) {
			$status &= CWEnumGameStatus::Base;
		}
		return $status;
	}

	public function updateTimeDiff()
	{
		if (!empty($modified = $this->modified_utc)) {
			$datetime_utc = strtotime($modified);
			$now = time();
			$diff = $now - $datetime_utc;
		} else {
			$diff = 0;
		}
		return $diff;
	}

	/* ---------------------------------------------- */
	/* Param Values
	/* ---------------------------------------------- */
	public function paramValue($key, $default = null)
	{
		return (isset($this->params[$key])) ? $this->params[$key] : $default;
	}

	/* ---------------------------------------------- */
	/* Memebers
	/* ---------------------------------------------- */
	public function evid()
	{
		return $this->paramValue('EVID');
	}

	public function type()
	{
		return (int)$this->paramValue('type', 0);
	}

	public function localStartDate()
	{
		return $this->paramValue('start_local');
	}

	public function localFinishedDate()
	{
		return $this->paramValue('finished_local');
	}

	public function sort()
	{
		return $this->paramValue('sort');
	}

	public function title()
	{
		return $this->paramValue('title');
	}

	public function game()
	{
		return $this->paramValue('GAID');
	}

	public function limit()
	{
		return $this->paramValue('LIID');
	}

	/* ---------------------------------------------- */
	/* Render
	/* ---------------------------------------------- */
	public function renderShortCode(&$attr)
	{
		return $this->render($attr);
	}

	public function render($attr)
	{
		$file = CWPATH . "templates/" . $this->template . ".php";
		if (!file_exists($file)) {
			return "ERROR";
		}
		ob_start();
		include($file);
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public function statusAsText()
	{
		if (!($status = $this->status())) {
			return "";
		}
		if ($status & CWEnumGameStatus::TimeOut) {
			$v = CWEnumGameStatus::TimeOut;
		} elseif ($status & CWEnumGameStatus::PauseSeating) {
			$v = CWEnumGameStatus::PauseSeating;
		} elseif ($status & CWEnumGameStatus::PauseClock) {
			$v = CWEnumGameStatus::PauseClock;
		} else {
			$v = $status & CWEnumGameStatus::Base;
		}
		$list = CWBaseGame::listOfStatusNames();
		return __($this->valueForKey($list, $v, ""), CWSLUG);
	}

	public function statusAsImage()
	{
		if (!($status = $this->status())) {
			return "";
		}
		if ($status & CWEnumGameStatus::TimeOut) {
			$v = CWEnumGameStatus::TimeOut;
		} elseif ($status & CWEnumGameStatus::PauseSeating) {
			$v = CWEnumGameStatus::PauseSeating;
		} elseif ($status & CWEnumGameStatus::PauseClock) {
			$v = CWEnumGameStatus::PauseClock;
		} else {
			$v = $status & CWEnumGameStatus::Base;
		}
		$list = CWBaseGame::listOfStatusNames();
		$name = __($this->valueForKey($list, $v, ""), CWSLUG);
		$list = $this->listOfStatusImageNames();
		$img = $this->valueForKey($list, $v, "");
		$src = CWURL . "imgs/control/control-" . $img . "@2x.png";
		return '<div><image class="cw-game-status-img" src="' . $src . '" alt="' . $name . '"/> ' . $name . '</div>';
	}

	public function pubAsText()
	{
		$v = $this->paramValue('pub', 0);
		$list = $this->listOfPublications();
		return __($this->valueForKey($list, $v, ""), CWSLUG);
	}

	public function formattedParamValue($key, $args = null)
	{
		switch ($key) {
			case 'color':
				$v = $this->paramValue($key);
				break;
			case 'title':
				if (null !== ($v = $this->paramValue($key))) {
					$v = esc_html($v);
					if (null !== ($page_id = $this->valueForKey($args, 'page_id'))) {
						$id = $this->id();
						$url = get_permalink($page_id);
						$v = "<a class=\"cw-title\" href=\"{$url}?id={$id}\">{$v}</a>";
					}
				}
				break;
			case 'pulldown':
				$v = true;
				break;
			case 'start_date_local':
				switch ($this->type()) {
					case CWEnumGameType::SitGo: // Sit&Go
						if (null !== ($v = $this->paramValue('countseats'))) {
							if (0 == $v) {
								$v = "-";
							} else {
								$v = sprintf(_n('%s Entry', '%s Entries', $v, CWSLUG), number_format_i18n($v));
							}
						}
						break;
					case CWEnumGameType::Ring: // Ring
						$v = "Ring";
						break;
					case CWEnumGameType::Dated: // Dated
						if (null !== ($v = $this->localStartDate())) {
							$t = strtotime($v);
							$format = get_option('date_format');
							$v = date_i18n($format, $t);
						}
						break;
					default:
						$v = null;
				}
				break;
			case 'start_time_local':
				switch ($this->type()) {
					case CWEnumGameType::SitGo:
						if (($v = $this->paramValue('countseats')) !== null) {
							if (0 == $v) {
								$v = "-";
							} else {
								$v = sprintf(_n('%s Entry', '%s Entries', $v, CWSLUG), number_format_i18n($v));
							}
						}
						break;
					case CWEnumGameType::Ring:
						$v = "∞";
						break;
					case CWEnumGameType::Dated:
						if (null !== ($v = $this->localStartDate())) {
							$t = strtotime($v);
							$format = get_option('time_format');
							$v = date_i18n($format, $t);
						}
						break;
					default:
						$v = null;
				}
				break;
			case 'start_date_time_local':
				switch ($this->type()) {
					case CWEnumGameType::SitGo:
						if (null !== ($v = $this->paramValue('countseats'))) {
							if (0 == $v) {
								$v = "-";
							} else {
								$v = sprintf(_n('%s Entry', '%s Entries', $v, CWSLUG), number_format_i18n($v));
							}
						}
						break;
					case CWEnumGameType::Ring:
						$v = "Ring";
						break;
					case CWEnumGameType::Dated:
						if (null !== ($v = $this->localStartDate())) {
							$t = strtotime($v);
							$format = get_option('date_format') . ', ' . get_option('time_format');
							$v = date_i18n($format, $t);
						}
						break;
					default:
						$v = null;
				}
				break;
			case 'finished_time_local':
				if (null !== ($v = $this->localFinishedDate())) {
					$t = strtotime($v);
					$format = get_option('time_format');
					$v = date_i18n($format, $t);
				} else {
					$v = null;
				}
				break;
			case 'total_time':
				if (null !== ($v = $this->paramValue('ttime'))) {
					$v = $this->formatDuration($v);
				} else {
					$v = "";
				}
				break;
			case 'status':
				$v = $this->statusAsText();
				break;
			case 'pub':
				$v = $this->pubAsText();
				break;
			case 'status.img':
				$v = $this->statusAsImage();
				break;
			case 'totalens':
			case 'countens':
			case 'countpls':
			case 'countwas':
			case 'counttables':
			case 'countseats':
				if (null !== ($v = $this->paramValue($key))) {
					$v = number_format_i18n($v);
				} else {
					$v = 0;
				}
				break;
			case 'remaincountlls':
				if ($this->paramValue('allowlls')) {
					$count = $this->paramValue('countlls');
					$remain = $this->paramValue('remainlls');
					if ($remain > 0 && $count >= $remain) {
						$v = "{$remain} / {$count}";
					} else {
						$v = "-";
					}
				} else {
					$v = null;
				}
				break;
			case 'freeseats':
				$seats = $this->paramValue('countseats');
				$players = $this->paramValue('countpls');
				if (null === $seats || null === $players || $players > $seats) {
					$v = "-";
				} else {
					$v = number_format_i18n($seats - $players);
				}
				break;
			case 'pcol':
				if (null !== ($v = $this->paramValue($key))) {
					esc_html($v);
				}
				break;
			case 'more':
			case 'more.flat':
				if (isset($args['page_id'])) {
					$page_id = $args['page_id'];
					$id = $this->id();
					$url = get_permalink($page_id);
					$title = isset($args['title']) ? esc_html($args['title']) : "More&hellip;";
					$flat = 'more.flat' == $key ? " cw-btn-flat" : "";
					$v = "<a href=\"{$url}?id={$id}\" class=\"cw-btn{$flat} cw-btn-success\">{$title}</a>";
				} else {
					$v = null;
				}
				break;
			default:
				if (null !== ($v = $this->paramValue($key))) {
					$v = esc_html($v);
				}
		}
		return $v;
	}

	public function formattedEntryValue($key, &$entry)
	{
		switch ($key) {
			case 'tableseat':
				if (!empty($this->valueForKey($entry, 'playing', 0))) {
					if ($table = $this->valueForKey($entry, 'table')) {
						$seat = $this->valueForKey($entry, 'seat', '');
						$v = esc_html($table) . " / " . esc_html($seat);
					} else {
						$v = "";
					}
				} else if ($place = $this->valueForKey($entry, 'place')) {
					$v = "Out " . $place;
				} else {
					$v = "DNQ";
				}
				break;
			case 'winnings': // All winnings, all currencies listed with <br>
				$v = "";
				$winnings = $this->valueForKey($entry, 'winnings');
				if ('array' == gettype($winnings) && count($winnings) > 0) {
					for ($c = 0; $c < 5; $c++) {
						if (!empty($win = $this->valueForKey($winnings, $c))) {
							if (0 < strlen($v)) {
								$v .= "<br>";
							}
							$w = $this->valueForKey($win, 'value');
							$v .= $this->formatValueWithCurrency($w, $c);
						}
					}
				} else {
					for ($c = 0; $c < 5; $c++) {
						if (!empty($w = $this->valueForKey($entry, 'winnings' . $c))) {
							if (0 < strlen($v)) {
								$v .= "<br>";
							}
							$v .= $this->formatValueWithCurrency($w, $c);
						}
					}
				}
				break;
			case 'bounty.rewards': // All bounties, all currencies listed with <br>
				$v = "";
				$bounties = $this->valueForKey($entry, 'bounties');
				if ('array' == gettype($bounties) && count($bounties) > 0) {
					for ($c = 0; $c < 5; $c++) {
						if (!empty($bounty = $this->valueForKey($bounties, $c))) {
							if (0 < strlen($v)) {
								$v .= "<br>";
							}
							$w = $this->valueForKey($bounty, 'value');
							$v .= $this->formatValueWithCurrency($w, $c);
						}
					}
				}
				break;
			case 'bounty.winnings': // All bounties, all currencies listed with <br>
				$v = "";
				$bounties = $this->valueForKey($entry, 'bounties');
				if ('array' == gettype($bounties) && count($bounties) > 0) {
					for ($c = 0; $c < 5; $c++) {
						if (!empty($bounty = $this->valueForKey($bounties, $c))) {
							$w = doubleval($this->valueForKey($bounty, 'win', 0));
							if ($w > 0) {
								if (0 < strlen($v)) {
									$v .= "<br>";
								}
								$v .= $this->formatValueWithCurrency($w, $c);
							}
						}
					}
				}
				break;
			default:
				if (null === ($v = $this->valueForKey($entry, $key))) {
					return "";
				}
				switch ($key) {
					case 'stack':
						$playing = (int)$this->valueForKey($entry, 'playing', 0);
						$v = $playing ? number_format_i18n($v) : "";
						break;
					case 'winnings0':
						$v = $this->formatValueWithCurrency($v);
						break;
					case 'winnings1':
						$v = $this->formatValueWithCurrency($v, 1);
						break;
					case 'winnings2':
						$v = $this->formatValueWithCurrency($v, 2);
						break;
					case 'winnings3':
						$v = $this->formatValueWithCurrency($v, 3);
						break;
					case 'winnings4':
						$v = $this->formatValueWithCurrency($v, 4);
						break;
					default:
						$v = $v ? esc_html($v) : "";
				}
		}

		return $v;
	}

	public function formattedSeatValue($key, &$seat)
	{
		switch ($key) {
			case 'SENO':
				$v = (isset($seat['SEID'])) ? intval($seat['SEID']) : 0;
				$v = number_format_i18n($v + 1);
				break;
			default:
				if (null === ($v = $this->valueForKey($seat, $key))) {
					return "";
				}
				switch ($key) {
					case 'stack':
						$v = number_format_i18n($v);
						break;
					case 'ensign':
						$path = "imgs/icons/{$v}-icon.gif";
						if (file_exists(CWPATH . $path)) {
							$src = CWURL . $path;
							$v = "<img style=\"margin: auto 0 auto 0;\" src=\"{$src}\" title=\"{$v}\" alt=\"{$v}\">";
						} else {
							$v = esc_html($v);
						}
						break;
					default:
						$v = $v ? esc_html($v) : "";
				}
		}

		return $v;
	}
}
