<?php
defined('ABSPATH') or die('No direct access');
class CWCashGame extends CWBaseGame
{
	public function __construct($data)
	{
		parent::__construct('cashgame', 'CGID', $data);
	}

	public function loadData($attr)
	{
		global $wpdb;
		$data = $wpdb->get_row("SELECT  `ID`,`LCID`,`modified_utc`,`params`,`entries`,`tables`,`waits` FROM `{$wpdb->prefix}cw_cashgames` WHERE `ID` = {$attr}", ARRAY_A);
		return $data;
	}

	/* ---------------------------------------------- */
	/* Memebers
	/* ---------------------------------------------- */
	public function statusAsText()
	{
		if (!($status = $this->status())) {
			return "";
		}
		if ($status & (CWEnumGameStatus::Initiate|CWEnumGameStatus::Register)) {
			$list = CWBaseGame::listOfStatusNames();
			$v = __($this->valueForKey($list, CWEnumGameStatus::Interest, ""), CWSLUG);
		} else {
			$v = parent::statusAsText();
		}
		return $v;
	}

	/* ---------------------------------------------- */
	/* Render
	/* ---------------------------------------------- */
	public function formattedParamValue($key, $args = null)
	{
		switch ($key) {
			case 'counttablenames':
				$v = parent::formattedParamValue('counttables', $args);
				$tables = $this->tables;
				if (!empty($tables) && 0 < count($tables)) {
					$v .= " (";
					$count = 0;
					foreach ($tables as $table) {
						if (0 < $count) {
							$v .= ", ";
						}
						$v .= $table['name'] ?? "";
						$count++;
					}
					$v .= ")";
				}
				break;
			case 'minmax':
				$min = $this->paramValue('cmin');
				$max = $this->paramValue('cmax');
				if (0 < $min || 0 < $max) {
					$v = "";
					if (0 < $min) {
						$v .= "Min ";
						$v .= $this->formatValueWithCurrency($min);
					}
					if (0 < $max) {
						if (strlen($v)) $v .= " - ";
						$v .= "Max ";
						$v .= $this->formatValueWithCurrency($max);
					}
				} else {
					$v = null;
				}
				break;
			case 'reg':
			case 'reg.level':
				$status = $this->status(true);
				switch ($status) {
					case CWEnumGameStatus::Initiate:
					case CWEnumGameStatus::Register:
						$v = "<div class=\"cw-bullet cw-bg-interest cw-me-1\"></div>Interest";
						break;
					case CWEnumGameStatus::Seating:
						$v = "<div class=\"cw-bullet cw-bg-seating cw-me-1\"></div>Seating";
						break;
					case CWEnumGameStatus::Running:
						$v = "<div class=\"cw-bullet cw-bg-open cw-me-1\"></div>Running";
						break;
					default:
						$v = "<div class=\"cw-bullet cw-bg-closed cw-me-1\"></div>Closed";
				}
				break;
			default:
				$v = parent::formattedParamValue($key, $args);
		}
		return $v;
	}
}
