<?php
defined('ABSPATH') or die('No direct access');
class CWLeaderboard extends CWBase
{
	protected $list;
	public $type;
	public $cur_id;
	public $cur_title;
	public $cur_count;
	public $search_bar;
	public $id;

	public function __construct($id)
	{
		$this->id = $id;
	}

	/* ---------------------------------------------- */
	/* Override
	/* ---------------------------------------------- */
	public function loadData($attr)
	{
		if (!$this->list) {
			global $wpdb;
			$options = get_option("cw_options_leaderboard");
			$type = $this->valueForKey($attr, 'type', 'stat');
			if ('stat' != $type || 'earn' != $type || 'bal' != $type) {
				$type = 'stat';
			}
			$this->type = $type;
			$cur_id = (int)$this->valueForKey($attr, 'cur_id', 0);
			if (0 > $cur_id || 4 < $cur_id) {
				$cur_id = 0;
			}
			$this->cur_id = $cur_id;
			$this->cur_title = $this->valueForKey($attr, 'cur_title');
			$this->cur_count = $this->valueForKey($attr, 'cur_count');
			$nozero = $this->valueForKey($attr, 'nozero');
			$limit = $this->valueForKey($attr, 'limit');
			$field = $this->type . $this->cur_id;
			$search_bar = $this->valueForKey($options, 'search_bar', 0);
			if ($search_bar) {
				$search_bar = $this->valueForKey($attr, 'search_bar', ($limit != null && ($limit >= 100 || $limit <= 20)) ? 0 : 1);
			}
			$this->search_bar = $search_bar;

			// Compose where clause
			$where = $field . "count > 0";
			if (($LCID = $this->valueForKey($attr, 'lcid')) !== null) {
				$where .= " AND `LCID` = {$LCID}";
			}
			if ($nozero) {
				$where .= " AND {$field}value > 0";
			}
			// Compose order by
			$order = " ORDER BY `{$field}value` DESC, `{$field}count`, `dname`";
			if (empty($limit)) {
				$limit = "";
			} else {
				$limit = " LIMIT " . $limit;
			}
			$rows = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}cw_players` WHERE {$where}{$order}{$limit}", ARRAY_A);
			$this->list = $rows;
		}
		return $this->list;
	}

	/* ---------------------------------------------- */
	/* Render
	/* ---------------------------------------------- */
	public function renderShortCode(&$attr)
	{
		return $this->render($attr);
	}

	public function render($attr)
	{
		$file = CWPATH . "templates/leaderboard.php";
		if (!file_exists($file)) {
			return "ERROR: File missing!";
		}
		ob_start();
		include($file);
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	/* ---------------------------------------------- */
	/* Render Table List
	/* ---------------------------------------------- */
	public function countEnabledTableColumns(&$options)
	{
		$column_idx = 0;
		if ($options) {
			foreach ($options as $option) {
				if ($this->valueForKey($option, 'enabled', 0)) {
					$column_idx++;
				}
			}
		}
		return $column_idx;
	}

	public function renderSearchBar(&$list, &$options)
	{
		if (!empty($list) && $this->search_bar) {
			$placeholder = esc_html($this->valueForKey($options, 'search_placeholder', "Search player..."));
			echo "<div class=\"cw-text-center\">";
			echo "<div class=\"cw-search-container\">";
			echo "<input type=\"text\" class=\"cw-search-input\" id=\"cw-search-input-{$this->id}\" placeholder=\"" . $placeholder . "\">";
			echo "<div class=\"cw-clear-div\">";
			echo "<button id=\"cw-clear-input-button-{$this->id}\" type=\"reset\" class=\"cw-clear-input-button\">x</button>";
			echo "</div>";
			echo "</div>";
			echo "</div>" . PHP_EOL;
		}
	}

	public function renderTableEmpty(&$list, &$options)
	{
		if ($empty = empty($list)) {
			$text = esc_html($this->valueForKey($options, 'empty_list', "Empty List!"));
			echo "<tr><th class=\"cw-text-center\">" . $text . "</th></tr>" . PHP_EOL;
		}
		return $empty;
	}
	
	public function renderTableHeader(&$list, &$options, $nrof_columns)
	{
		if ($this->renderTableEmpty($list, $options)) {
			return;
		}
		if (!isset($options['table_headers']) || !$options['table_headers']) {
			return;
		}
		echo "<tr class=\"cw-tr-seps\">" . PHP_EOL;
		if ($options) {
			$flex_column = false;
			$column_idx = 0;
			foreach ($options as $key => $option) {
				if ($this->valueForKey($option, 'enabled', 0)) {
					$collapse = $this->valueForKey($option, 'collapse');
					$classes = ($collapse) ? " cw-hide-{$collapse}" : "";
					if ($key == 'curvalue' && !empty($this->cur_title)) {
						$title = esc_html($this->cur_title);
					} else if ($key == 'curcount' && !empty($this->cur_count)) {
						$title = esc_html($this->cur_count);
					} else {
						$title = esc_html($this->valueForKey($option, 'title', ""));
					}
					if (!$flex_column && $column_idx + 1 == $nrof_columns) {
						$w = 0;
					} else {
						$w = (int)$this->valueForKey($option, 'width', 0);
					}
					if ($w == 0) $flex_column = true;
					$styles = "";
					if ($w > 0) {
						$styles .= " style=\"";
						if ($w > 0) {
							$styles .= "width:{$w}px;";
						}

						$styles .= "\"";
					}
					echo "<th class=\"cw-th-seps{$classes}\"{$styles}>{$title}</th>" . PHP_EOL;
				}
			}
		}
		echo "</tr>" . PHP_EOL;
	}

	public function renderTableBody(&$list, &$options, $nrof_columns)
	{
		if (!empty($list)) {
			$row_idx = 0;
			foreach ($list as $pl) {
				$this->renderTableRow($pl, $options, $row_idx, $nrof_columns);
				$this->renderTableRowDetails($pl, $options, $row_idx, $nrof_columns);
				$row_idx++;
			}
		}
	}

	public function renderTableRow(&$pl, &$options, $row_idx, $nrof_columns)
	{
		// Render row with columns
		echo "<tr id=\"cw_tr_{$row_idx}\" class=\"cw-tr-seps cw-tr-searchable\">" . PHP_EOL;
		if ($options) {
			$flex_column = false;
			$column_idx = 0;
			foreach ($options as $key => $option) {
				if ($this->valueForKey($option, 'enabled', 0)) {
					if (($key == 'more' || $key == 'dname') && $this->valueForKey($options, 'details_page_id')) {
						$option['page_id'] = $options['details_page_id'];
						if ($key == 'more') {
							$key = 'more.flat';
						}
						$collapse = null;
					} else {
						$collapse = $this->valueForKey($option, 'collapse');
					}
					if ($collapse && $key != 'rank') {
						$classes = " cw-hide-{$collapse}";
					} else {
						$classes = "";
					}
					echo $this->renderTableCell($pl, $key, $option, $row_idx, $column_idx, $nrof_columns, $flex_column, $classes) . PHP_EOL;
					$column_idx++;
				}
			}
		}
		echo "</tr>" . PHP_EOL;
	}

	public function renderTableCell(&$pl, $key, $option, $row_idx, $column_idx, $nrof_columns, &$flex_column, $classes)
	{
		$v = $this->formattedPlayerValue($key, $pl, $row_idx, "");
		if (!$flex_column && $column_idx + 1 == $nrof_columns) {
			$w = 0;
		} else {
			$w = (int)$this->valueForKey($option, 'width', 0);
		}
		if ($w == 0) $flex_column = true;
		if ($w > 0 || ($row_idx >= 0 && $row_idx <= 2 && $key == 'rank')) {
			$styles = " style=\"";
			if ($w > 0) {
				$styles .= "width:{$w}px;";
			}
			if ($key == 'rank') {
				switch ($row_idx) {
					case 0:
						$styles .= "background-color:goldenrod;";
						break;
					case 1:
						$styles .= "background-color:silver;";
						break;
					case 2:
						$styles .= "background-color:#bf8970;";
						break;
				}
			}
			$styles .= "\"";
		} else {
			$styles = "";
		}
		$classes .= " cw-text-" . $this->valueForKey($option, 'align', 'center');
		$s = "<td class=\"cw-td-seps{$classes}\"{$styles}>{$v}</td>";
		return $s;
	}

	public function renderTableRowDetails(&$pl, &$options, $row_idx, $nrof_columns)
	{
		// Create details
		$drawer_open = $this->valueForKey($options, 'drawer');
		$classes = $drawer_open ? " cw-show-{$drawer_open}" : "";
		echo "<tr class=\"cw-tr-seps-drawer\"><td class=\"cw-td-seps-drawer\" colspan=\"{$nrof_columns}\">" . PHP_EOL;
		if ($drawer_open) {
			echo "<div id=\"cw_tr_details_{$this->id}_{$row_idx}\" class=\"cw-tr-seps-details{$classes}\">" . PHP_EOL;
			// Name
			if (isset($options['dname']['detail']) && $options['dname']['detail']) {
				if (($v = $this->formattedPlayerValue('dname', $pl)) !== null) {
					echo "<p>{$v}</p>" . PHP_EOL;
				}
			}

			echo "<div class=\"cw-row\">" . PHP_EOL;

			// Left side
			echo "<!-- Details -->" . PHP_EOL;
			echo "<div class=\"cw-col-12\">" . PHP_EOL;
			echo "<table class=\"cw-table-details\">" . PHP_EOL;
			$detail_count = 0;
			foreach ($options as $key => $option) {
				if (gettype($option) == 'array') {
					if ($key != 'dname' && $key != 'more') {
						if ($this->valueForKey($option, 'detail')) {
							if (($l = $this->valueForKey($option, 'title', "")) != "") {
								$l .= ":&nbsp;&nbsp;";
							}
							if (($v = $this->formattedPlayerValue($key, $pl, $detail_count)) !== null) {
								$s = "<tr><td class=\"cw-label\">{$l}</td><td class=\"cw-cell\">{$v}</td></tr>";
								echo $s . PHP_EOL;
								$detail_count++;
							}
						}
					}
				}
			}
			echo "</table>" . PHP_EOL;

			echo "</div>" . PHP_EOL;
			echo "<!-- End Details -->" . PHP_EOL;
			// Left side End

			echo "</div>" . PHP_EOL;
			echo "</div>" . PHP_EOL; // End Details
		}

		echo "</td></tr>" . PHP_EOL;
	}

	public function formattedPlayerValue($key, &$pl, $row_idx = 0, $default = null, $args = null)
	{
		switch ($key) {
			case 'rank':
				$v = "<strong>" . number_format_i18n($row_idx + 1) . "</strong>";
				break;
			case 'ensign':
				if ($ensign = $this->valueForKey($pl, 'ensign')) {
					$path = "imgs/icons/{$ensign}-icon.gif";
					if (file_exists(CWPATH . $path)) {
						$src = CWURL . $path;
						$v = "<img style=\"margin: auto 0 auto 0;\" src=\"{$src}\" title=\"{$ensign}\" alt=\"{$ensign}\">";
					} else {
						$v = esc_html($ensign);
					}
				} else {
					$v = $default;
				}
				break;
			case 'ptime':
				$v = $this->valueForKey($pl, $this->type . $this->cur_id . "ptime", 0);
				$v = $this->formatDuration($v);
				break;
			case 'curcount':
				$v = $this->valueForKey($pl, $this->type . $this->cur_id . "count", 0);
				$v = number_format_i18n($v);
				break;
			case 'curvalue':
				$v = $this->valueForKey($pl, $this->type . $this->cur_id . "value", 0);
				$v = $this->formatValueWithCurrency($v, $this->cur_id);
				break;
			case 'more':
			case 'more.flat':
				if (isset($args['page_id'])) {
					$page_id = $args['page_id'];
					$id = $this->valueForKey($pl, 'PLID');
					$url = get_permalink($page_id);
					$title = isset($args['title']) ? esc_html($args['title']) : "More&hellip;";
					$flat = $key == 'more.flat' ? " cw-btn-flat" : "";
					$v = "<a href=\"{$url}?id={$id}\" class=\"cw-btn{$flat} cw-btn-success\">{$title}</a>";
				} else {
					$v = null;
				}
				break;
			default:
				$v = $this->valueForKey($pl, $key, $default);
				if (!empty($v)) {
					$v = esc_html($v);
				}
		}

		return $v;
	}
}
