<?php
defined('ABSPATH') or die('No direct access');
class CWRestApi
{
	use CWFoundationTrait;

	/* ---------------------------------------------- */
	/* Rest API
	/* ---------------------------------------------- */
	public function register()
	{
		add_action('rest_api_init', [$this, 'initRestApi']);
	}

	public function initRestApi()
	{
		register_rest_route('casinoware/v1', 'rest-api', ['methods' => 'POST', 'callback' => [$this, 'handleRequest'], 'permission_callback' => '__return_true']);
	}

	public function handleRequest($req)
	{
		// Get the api settings
		$options = get_option('cw_options_api');
		if (!$options) {
			return new WP_REST_Response(['result' => -100, 'msg' => "WordPress API-Settings not found!"], 200);
		}
		// Get the json data from the request
		$data = $req->get_json_params();
		// Sender informaton
		if (!isset($data['sender'])) {
			return new WP_REST_Response(['result' => -500, 'msg' => "Missing Sender data block!"], 200);
		}
		$sender = $this->valueForKey($data, 'sender');
		// Check username
		if (!isset($sender['uname'])) {
			return new WP_REST_Response(['result' => -511, 'msg' => "Missing Username in CasinoWare Sender data block!"], 200);
		}
		if (!isset($options['sender_uname'])) {
			return new WP_REST_Response(['result' => -111, 'msg' => "Username not set in WordPress API-Settings!"], 200);
		}
		if ($sender['uname'] !== $options['sender_uname']) {
			return new WP_REST_Response(['result' => -101, 'msg' => "Incorrect Username!"], 200);
		}
		// Check password
		if (!isset($sender['password'])) {
			return new WP_REST_Response(['result' => -512, 'msg' => "Missing Password in CasinoWare Sender data block!"], 200);
		}
		if (!isset($options['sender_password'])) {
			return new WP_REST_Response(['result' => -112, 'msg' => "Password not set in WordPress API-Settings!"], 200);
		}
		if ($sender['password'] !== $options['sender_password']) {
			return new WP_REST_Response(['result' => -102, 'msg' => "Incorrect Password!"], 200);
		}
		// Check apikey
		if (!isset($sender['apikey'])) {
			return new WP_REST_Response(['result' => -513, 'msg' => "Missing API-Key in CasinoWare Sender data block!"], 200);
		}
		if (!isset($options['sender_apikey'])) {
			return new WP_REST_Response(['result' => -113, 'msg' => "API-Key not set in WordPress API-Settings!"], 200);
		}
		if ($sender['apikey'] !== $options['sender_apikey']) {
			return new WP_REST_Response(['result' => -103, 'msg' => "Incorrect API-Key!"], 200);
		}
		// Check system version
		if (!isset($sender['system'])) {
			return new WP_REST_Response(['result' => -514, 'msg' => "Missing system version in CasinoWare Sender data block!"], 200);
		}
		if (version_compare($sender['system'], '2.4') < 0) {
			return new WP_REST_Response(['result' => -104, 'msg' => "CasinoWare version 2.4 needed!"], 200);
		}
		// Location or venue ID
		$LCID = isset($sender['LCID']) ? $sender['LCID'] : null;
		// General update data
		if (!($general = $this->valueForKey($data, 'general'))) {
			return new WP_REST_Response(['result' => -500, 'msg' => "Missing General data block!"], 200);
		}

		// Update tournaments
		$tablename = 'tournaments';
		if ($tts = ($this->valueForKey($data, $tablename))) {
			$count_total_tts = count($tts);
			$error = $this->updateGames($tts, $general, $LCID, 'TTID', $tablename, $count_tts);
			if ($error !== null) return $error;
		} else {
			$count_total_tts = 0;
			$count_tts = 0;
		}
		// Update cash games
		$tablename = 'cashgames';
		if ($cgs = ($this->valueForKey($data, $tablename))) {
			$count_total_cgs = count($cgs);
			$error = $this->updateGames($cgs, $general, $LCID, 'CGID', $tablename, $count_cgs);
			if ($error !== null) return $error;
		} else {
			$count_total_cgs = 0;
			$count_cgs = 0;
		}
		// Update players
		$tablename = 'players';
		if ($pls = ($this->valueForKey($data, $tablename))) {
			$count_total_pls = count($pls);
			$error = $this->updatePlayers($pls, $general, $LCID, 'PLID', $tablename, $count_pls);
			if ($error !== null) return $error;
		} else {
			$count_total_pls = 0;
			$count_pls = 0;
		}

		$type = $this->valueForKey($general, 'update', 'unknown');
		return new WP_REST_Response(['result' => 0, 'msg' => "{$type} TTs {$count_tts}/{$count_total_tts}, CGs {$count_cgs}/{$count_total_cgs}, Players {$count_pls}/{$count_total_pls}"], 200);
	}

	protected function updatePlayers($pls, $general, $LCID, $idfield, $tablename, &$count)
	{
		// Database access
		global $wpdb;

		$count = 0;
		$table = "{$wpdb->prefix}cw_{$tablename}";
		$fieldnames = [
			'LCID', 'modified_utc', 'modified_local',
			$idfield, 'address', 'ensign', 'anonymous', 'MEID', 'dname', 'fname', 'lname', 'nname', 'email', 'mdno', 'fpp',
			'sctime', 'sctables', 'tctime', 'tctables',
			'bal0value', 'bal1value', 'bal2value', 'bal3value', 'bal4value',
			'stat0value', 'stat0count', 'stat0ptime', 'stat1value', 'stat1count', 'stat1ptime', 'stat2value', 'stat2count', 'stat2ptime', 'stat3value', 'stat3count', 'stat3ptime', 'stat4value', 'stat4count', 'stat4ptime',
			'earn0value', 'earn0count', 'earn0ptime', 'earn1value', 'earn1count', 'earn1ptime', 'earn2value', 'earn2count', 'earn3ptime', 'earn3value', 'earn3count', 'earn3ptime', 'earn4value', 'earn4count', 'earn4ptime'
		];
		$fieldtypes = [
			'%d', '%s', '%s',
			'%d', '%d', '%s', '%d', '%d', '%s', '%s', '%s', '%s', '%s', '%s', '%d',
			'%d', '%d', '%d', '%d',
			'%f', '%f', '%f', '%f', '%f',
			'%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d',
			'%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d'
		];

		$mod_utc = $this->valueForKey($general, 'datetime_utc');
		$mod_local = $this->valueForKey($general, 'datetime_local');

		foreach ($pls as $pl) {
			if (isset($pl[$idfield])) {
				// Prepare
				if ($bals = $this->valueForKey($pl, 'balances')) {
					foreach ($bals as $key => $bal) {
						$v = floatval($this->valueForKey($bal, 'value', 0));
						$f = 'bal' . $key . 'value';
						$pl[$f] = $v;
					}
				}
				$this->preparePlayerWinnings($pl, 'stat', 'swins');
				$this->preparePlayerWinnings($pl, 'earn', 'twins');
				// Set data
				$data_vals = [];
				$data_vals['LCID'] = $LCID;
				$data_vals['modified_utc'] = $mod_utc;
				$data_vals['modified_local'] = $mod_local;
				for ($i = 3; $i < count($fieldnames); $i++) {
					$f = $fieldnames[$i];
					$dt = $fieldtypes[$i];
					$v = $this->valueForKey($pl, $f, ('%s' == $dt) ? null : 0);
					$data_vals[$f] = $v;
				}
				if ($error = $this->setRow($table, $LCID, $idfield, $data_vals, $fieldtypes)) {
					return $error;
				}
				$count++;
			} else {
				// return new WP_REST_Response(['result' => -108, 'msg' => "Error: ID field '{$idfield}' missing in player's data."], 200);
			}
		}

		return null;
	}

	protected function preparePlayerWinnings(&$pl, $key, $fieldname)
	{
		if ($wins = $this->valueForKey($pl, $fieldname)) {
			foreach ($wins as $cur => $win) {
				$f = $key . $cur;
				$v = floatval($this->valueForKey($win, 'value', 0));
				$pl[$f . 'value'] = $v;
				$v = intval($this->valueForKey($win, 'count', 0));
				$pl[$f . 'count'] = $v;
				$v = intval($this->valueForKey($win, 'ptime', 0));
				$pl[$f . 'ptime'] = $v;
			}
		}
	}

	protected function updateGames($bgs, $general, $LCID, $idfield, $tablename, &$count)
	{
		// Database access
		global $wpdb;

		$count = 0;
		$table = "{$wpdb->prefix}cw_{$tablename}";
		// Lists
		$bg_lists = ['entries', 'tables', 'waits']; // Lists for all games
		$tt_lists = ['levels', 'payouts']; // Lists only for tournaments

		foreach ($bgs as $bg) {
			if (isset($bg[$idfield])) {
				$data_vals = [];
				$data_vals[$idfield] = (int)$this->valueForKey($bg, $idfield);
				$data_vals['modified_utc'] = $this->valueForKey($general, 'datetime_utc');
				$data_vals['modified_local'] = $this->valueForKey($general, 'datetime_local');
				$data_vals['LCID'] = $LCID;
				$data_vals['GGID'] = $this->valueForKey($bg, 'GGID', 0);
				$data_vals['EVID'] = $this->valueForKey($bg, 'EVID', null);
				$data_vals['status'] = isset($bg['status']) ? (((int) $bg['status']) & CWEnumGameStatus::Base) : 0;
				$data_vals['pub'] = $this->valueForKey($bg, 'pub', CWEnumGamePublish::None);
				$data_vals['type'] = $this->valueForKey($bg, 'type', CWEnumGameType::None);
				$data_vals['league'] = $this->valueForKey($bg, 'league', null);
				$data_vals['start_utc'] = $this->valueForKey($bg, ($data_vals['type'] === CWEnumGameType::SitGo ? 'started_utc' : 'start_utc'), null);
				$data_vals['start_local'] = $this->valueForKey($bg, ($data_vals['type'] === CWEnumGameType::SitGo ? 'started_local' : 'start_local'), null);
				$data_vals['title'] = $this->valueForKey($bg, 'title', null);
				$data_types = ['%d', '%s', '%s', '%d', '%d', '%d', '%d', '%d', '%d', '%s', '%s', '%s', '%s'];
				foreach ($bg_lists as $list) {
					if (isset($bg[$list])) {
						$data_vals[$list] = json_encode($bg[$list]);
						unset($bg[$list]);
					} else {
						$data_vals[$list] = null;
					}
					$data_types[] = '%s';
				}
				if ($tablename == 'tournaments') {
					foreach ($tt_lists as $list) {
						if (isset($bg[$list])) {
							$data_vals[$list] = json_encode($bg[$list]);
							unset($bg[$list]);
						} else {
							$data_vals[$list] = null;
						}
						$data_types[] = '%s';
					}
				}
				// Last but not least set the params
				$data_vals['params'] = json_encode($bg);
				$data_types[] = '%s';

				if ($error = $this->setRow($table, $LCID, $idfield, $data_vals, $data_types)) {
					return $error;
				}
				$count++;
			} else {
				// return new WP_REST_Response(['result' => -107, 'msg' => "Error: ID field '{$idfield}' missing in game's data."], 200);
			}
		}
		return null;
	}

	protected function setRow($table, $LCID, $idfield, &$data_vals, &$data_types)
	{
		// Database access
		global $wpdb;
		// Get the unique row ID if game already exists
		$q = $wpdb->prepare("SELECT ID from {$table} WHERE LCID = %d AND {$idfield} = %s;", $LCID, $data_vals[$idfield]);
		$r = $wpdb->get_row($q, ARRAY_A);
		if (!$r && !empty($wpdb->last_error)) {
			$error = $wpdb->last_error;
			return new WP_REST_Response(['result' => -105, 'msg' => "Error: {$error}"], 200);
		}
		if ($ID = $this->valueForKey($r, 'ID')) {
			// Update if we already have an ID
			$r = $wpdb->update($table, $data_vals, ['ID' => $ID], $data_types, ['%d']);
		} else {
			// Insert row
			$r = $wpdb->insert($table, $data_vals, $data_types);
		}
		if (!$r && !empty($wpdb->last_error)) {
			$error = $wpdb->last_error;
			return new WP_REST_Response(['result' => -106, 'msg' => "Error: {$error}"], 200);
		}
		return null;
	}
}
