<?php
defined('ABSPATH') or die('No direct access');
class CWTournament extends CWBaseGame
{
	public function __construct($data)
	{
		parent::__construct('tournament', 'TTID', $data);
	}

	public function loadData($attr)
	{
		global $wpdb;
		$data = $wpdb->get_row("SELECT `ID`,`LCID`,`modified_utc`,`params`,`entries`,`tables`,`waits`,`levels`,`payouts` FROM `{$wpdb->prefix}cw_tournaments` WHERE `ID` = {$attr}", ARRAY_A);
		return $data;
	}

	/* ---------------------------------------------- */
	/* Memebers
	/* ---------------------------------------------- */
	public function typepb()
	{
		return $this->paramValue('typepb', 0);
	}

	public function costs($name, $currency)
	{
		if ($costs = $this->paramValue('costs' . $name . $currency)) {
			$buyin = floatval($this->valueForKey($costs, 'cost', 0));
			$fee = floatval($this->valueForKey($costs, 'fee', 0));
			$total = $buyin + $fee;
		} else {
			$total = 0;
		}
		return $total;
	}

	public function totalpb($currency = 0)
	{
		$v = $this->costs('pb', $currency);
		if ($this->paramValue('allowbts')) {
			$v += $this->costs('bt', $currency);
		}
		return $v;
	}

	public function levelDuration($level = 0)
	{
		if (($levels = $this->levels) === null) {
			return null;
		}
		if (!isset($levels[0]['duration'])) {
			return null;
		}
		return intval($levels[0]['duration']);
	}

	public function buyInsAsBadges()
	{
		$append = false;
		$v = "";
		if ($this->paramValue('allowbus')) {
			if ($append) $v .= " ";
			$v .= "<span class=\"cw-badge cw-bg-buffet\">Buffet</span>";
			$append = true;
		}
		if ($this->paramValue('allowlls')) {
			if ($append) $v .= " ";
			$v .= "<span class=\"cw-badge cw-bg-lastlonger\">Last-Longer</span>";
			$append = true;
		}
		if ($this->paramValue('allowbss')) {
			if ($append) $v .= " ";
			$v .= "<span class=\"cw-badge cw-bg-bonus\">Bonus</span>";
			$append = true;
		}
		if ($this->paramValue('allowbts')) {
			if ($append) $v .= " ";
			$v .= "<span class=\"cw-badge cw-bg-bounty\">Bounty</span>";
			$append = true;
		}
		if ($this->paramValue('allowres')) {
			if ($append) $v .= " ";
			$n = $this->paramValue('maxres', 0);
			$nrof = $n > 0 ? $n . 'x' : "";
			$v .= "<span class=\"cw-badge cw-bg-reentry\">{$nrof}Re-Entry</span>";
			$append = true;
		}
		if ($this->paramValue('allowrbs')) {
			if ($append) $v .= " ";
			$n = $this->paramValue('maxrbs', 0);
			$nrof = $n > 0 ? $n . 'x' : "";
			$v .= "<span class=\"cw-badge cw-bg-rebuy\">{$nrof}Re-Buy</span>";
			$append = true;
		}
		if ($this->paramValue('allowaos')) {
			if ($append) $v .= " ";
			$n = $this->paramValue('maxaos', 0);
			$nrof = $n > 0 ? $n . 'x' : "";
			$v .= "<span class=\"cw-badge cw-bg-addon\">{$nrof}Add-On</span>";
			$append = true;
		}
		return $v;
	}

	public function chips()
	{
		return isset($this->params['costspb0']['chips']) ? $this->params['costspb0']['chips'] : null;
	}

	public function isRegistrationOpen()
	{
		$status = $this->status(true);
		// Primary Buy-In type: 0 - Standard, 1 - Freeroll, 2 - Limited/Final
		$typepb = $this->typepb();
		if (0 != $typepb && 1 != $typepb) {
			return false;
		}
		$level = $this->paramValue('level', 0); // Current level
		$xlevel = $this->paramValue('xlevel', 0); // Late registration till level
		return (CWEnumGameStatus::Register == $status || CWEnumGameStatus::Seating == $status || (CWEnumGameStatus::Running == $status && $level <= $xlevel));
	}

	public function formattedBlindsForLevel($level)
	{
		$s = "";
		if (empty($this->levels)) {
			return $s;
		}
		$lidx = $level - 1; // Current level
		$levels = $this->levels;
		if (0 > $lidx || $lidx >= count($levels)) {
			return $s;
		}
		$level = $levels[$lidx];
		if (!empty($level['smallblind'])) {
			$s .= $this->formattedLevelValue('smallblind', $level);
		}
		if (!empty($level['bigblind'])) {
			if (!empty($s)) {
				$s .= ' / ';
			}
			$s .= $this->formattedLevelValue('bigblind', $level);
		}
		if (!empty($level['ante'])) {
			if (!empty($s)) {
				$s .= ' ';
			}
			$s .= 'Ante ' . $this->formattedLevelValue('ante', $level);
		}
		if (!empty($level['bringin'])) {
			if (!empty($s)) {
				$s .= ' ';
			}
			$s .= 'Bring-In ' . $this->formattedLevelValue('bringin', $level);
		}

		return $s;
	}

	/* ---------------------------------------------- */
	/* Render
	/* ---------------------------------------------- */
	public function formattedParamValue($key, $args = null)
	{
		switch ($key) {
			case 'costspb0':
			case 'totalpb0':
				switch ($this->typepb()) {
					case 0: // Standard
						if (null !== ($v = $this->totalpb())) {
							$v = $this->formatValueWithCurrency($v);
							if (null !== ($ext = $this->paramValue('pbdetails'))) {
								$v .= " " . esc_html($ext);
							}
						}
						break;
					case 1: // Freeroll
						$v = "Freeroll";
						break;
					case 2: // Limited
						$v = "Limited";
						break;
					default:
						$v = "";
				}
				break;
			case 'guaranteed0':
			case 'prizepool0':
				if (null !== ($v = $this->paramValue($key))) {
					$v = $this->formatValueWithCurrency($v);
				}
				break;
			case 'level_duration':
				if (null !== ($v = $this->levelDuration())) {
					$v .= "'";
				}
				break;
			case 'additional_buyins':
				$v = $this->buyInsAsBadges();
				break;
			case 'startchips':
				if (null !== ($v = $this->chips())) {
					$v = number_format_i18n($v);
				}
				break;
			case 'countrbs':
			case 'countaos':
			case 'level':
			case 'ltime':
			case 'avgchips':
			case 'entirechips':
				$v = number_format_i18n($this->paramValue($key, 0));
				break;
			case 'blinds.current':
				$l = intval($this->paramValue('level', 1));
				if ($l < 1) {
					$l = 1;
				}
				$v = $this->formattedBlindsForLevel($l);
				break;
			case 'blinds.next':
				$l = intval($this->paramValue('level', 1)) + 1;
				if ($l < 2) {
					$l = 2;
				}
				$v = $this->formattedBlindsForLevel($l);
				break;
			case 'reg.level':
			case 'reg':
				$status = $this->status(true);
				// Primary Buy-In type: 0 - Standard, 1 - Freeroll, 2 - Limited/Final
				$typepb = $this->typepb();
				switch ($typepb) {
					case 0: // Standard
					case 1: // Freeroll
						if (CWEnumGameStatus::Initiate == $status) {
							$v = "<div class=\"cw-bullet cw-bg-secondary cw-me-1\" style=\"height:12px;width:12px;\"></div>Scheduled";
						} else {
							$level = $this->paramValue('level', 0); // Current level
							$xlevel = $this->paramValue('xlevel', 0); // Late registration till level
							if (CWEnumGameStatus::Register == $status || CWEnumGameStatus::Seating == $status || (CWEnumGameStatus::Running == $status && $level <= $xlevel)) {
								$v = "<div class=\"cw-bullet cw-bg-open cw-me-1\" style=\"height:12px;width:12px;\"></div>";
								if (0 < $xlevel && 'reg' != $key) {
									$v .= "OPEN (till level {$xlevel})";
								} else {
									$v .= "OPEN";
								}
							} else {
								$v = "<div class=\"cw-bullet cw-bg-closed cw-me-1\" style=\"height:12px;width:12px;\"></div>CLOSED";
							}
						}
						break;
					case 2: // Limited
						$v = "<div class=\"cw-bullet cw-bg-limited cw-me-1\" style=\"height:12px;width:12px;\"></div>FINAL";
						break;
					default:
						$v = null;
				}
				break;
			case 'stackupdate_local':
				if (null !== ($v = $this->paramValue($key))) {
					$t = strtotime($v);
					$format = get_option('time_format');
					$v = date_i18n($format, $t);
				}
				break;
			default:
				$v = parent::formattedParamValue($key, $args);
		}
		return $v;
	}

	public function formattedLevelValue($key, &$level)
	{
		if (($v = $this->valueForKey($level, $key)) === null) {
			return "";
		}
		switch ($key) {
			case 'duration':
			case 'colorupduration':
			case 'breakduration':
				$v = $v ? number_format_i18n($v) . "'" : "-";
				break;
			case 'info':
				$v = esc_html($v);
				break;
			default:
				if ($v) {
					$v = number_format_i18n($v);
				} else {
					$v = "-";
				}
		}

		return $v;
	}

	public function renderTableLevels($classes = "")
	{
		if (empty($this->levels)) {
			return;
		}
		$levels = $this->levels;
		echo "<!-- Levels -->" . PHP_EOL;
		echo "<table class=\"cw-table-list cw-table-levels {$classes}\">" . PHP_EOL;
		$level_count = 1;
		$columns = ['smallblind', 'bigblind', 'ante', 'duration'];
		echo "<thead>" . PHP_EOL;
		echo "<tr class=\"cw-tr\"><th class=\"cw-th\">Level</th><th class=\"cw-th\">Small Blind</th><th class=\"cw-th\">Big Blind</th><th class=\"cw-th\">Ante</th><th class=\"cw-th\">Min</th></tr>" . PHP_EOL;
		echo "</thead>" . PHP_EOL;
		echo "<tbody>" . PHP_EOL;
		$xlevel = $this->paramValue('xlevel', 0); // Late Reg
		foreach ($levels as $level) {
			$s = "<tr class=\"cw-tr\">";
			$s .= "<td class=\"cw-td cw-text-center\"><strong>$level_count</strong></td>";
			if ($level['smallblind'] > 0 || $level['bigblind'] > 0 || $level['ante'] > 0) {
				$col_count = 0;
			} else {
				$v = $this->formattedLevelValue('info', $level);
				$s .= "<td colspan=\"3\" class=\"cw-td cw-text-center\">$v</td>";
				$col_count = 3;
			}
			for (; $col_count < count($columns); $col_count++) {
				$column = $columns[$col_count];
				if (null !== ($v = $this->formattedLevelValue($column, $level))) {
					$s .= "<td class=\"cw-td cw-text-center\">$v</td>";
				} else {
					$s .= "<td></td>";
				}
			}

			$s .= "</tr>";
			echo $s  . PHP_EOL;
			if (0 < ($cu = $this->valueForKey($level, 'colorupduration', 0))) {
				echo '<tr class="cw-tr"><td class="cw-td cw-text-center cw-color-colorup cw-font-size-14" colspan="5">Color-Up ' . $cu . ' mins</td>' . PHP_EOL;
			}
			if (0 < ($br = $this->valueForKey($level, 'breakduration', 0))) {
				echo '<tr class="cw-tr"><td class="cw-td cw-text-center cw-color-break cw-font-size-14" colspan="5">Break ' . $br . ' mins</td>' . PHP_EOL;
			}
			if ($level_count == $xlevel) {
				echo '<tr class="cw-tr"><td class="cw-td cw-text-center cw-color-regend cw-font-size-14" colspan="5">End of Late Registration</td>' . PHP_EOL;
			}
			$level_count++;
		}
		echo "</tbody>" . PHP_EOL;
		echo "</table>" . PHP_EOL;
		echo "<!-- End Levels -->" . PHP_EOL;
	}

	public function formattedPayoutValue($key, &$pv)
	{
		if (null === ($v = $this->valueForKey($pv, $key))) {
			return "";
		}
		switch ($key) {
			case 'value':
				$v = $this->formatValueWithCurrency($v);
				break;
			default:
				$v = $v ? esc_html($v) : "";
		}

		return $v;
	}
}
