<?php
defined('ABSPATH') or die('No direct access');
class CWBaseList extends CWBase
{
	protected $list = null;
	protected $found = 0;
	protected $total = 0;
	protected $limit = 0;
	protected $pages = 0;
	protected $page = 0;
	public $game;
	public $figure;
	public $id;

	public function __construct($game, $figure, $id)
	{
		$this->game = $game;
		$this->figure = $figure;
		$this->id = $id;
	}

	/* ---------------------------------------------- */
	/* Functions
	/* ---------------------------------------------- */

	/* ---------------------------------------------- */
	/* Override
	/* ---------------------------------------------- */
	public function loadData($attr)
	{
	}

	protected function composeWhereWithAttr(&$attr)
	{
		$where = " WHERE ";
		switch ($this->figure) {
			case 'schedule':
				$today = time();
				$date = date("Y-m-d", $today);
				$where .= "`status` < " . CWEnumGameStatus::Finished;
				$where .= " AND `start_utc` >= '{$date}'";
				break;
			case 'results':
				$where .= "`status` = " . CWEnumGameStatus::Finished;
				break;
			default:
				$where .= "`status` < " . CWEnumGameStatus::Finished;
		}
		if (null !== ($LCID = $this->valueForKey($attr, 'lcid'))) $where .= " AND `LCID` = {$LCID}";
		if (null !== ($EVID = $this->valueForKey($attr, 'evid'))) $where .= " AND `EVID` = {$EVID}";
		if (null !== ($GGID = $this->valueForKey($attr, 'ggid'))) $where .= " AND `GGID` = {$GGID}";
		if (null !== ($type = $this->valueForKey($attr, 'type'))) $where .= " AND `type` = {$type}";
		if (null !== ($league = $this->valueForKey($attr, 'league'))) $where .= " AND `league` = '{$league}'";
		$where .= " AND (`pub` = " . CWEnumGamePublish::Once . " OR `pub` = " . CWEnumGamePublish::Always . " OR (`pub` = " . CWEnumGamePublish::OnRegistration . " AND `status` > " . CWEnumGameStatus::Initiate . "))";
		return $where;
	}

	protected function composeLimitWithAttr(&$attr)
	{
		$limit = $this->valueForKey($attr, 'limit');
		if (null === $limit) {
			$options = get_option("cw_options_{$this->game}_{$this->figure}");
			$limit = $this->valueForKey($options, 'limit');
		}
		if (empty($limit)) {
			return "";
		}
		$s = " LIMIT " . $limit;
		$page = get_query_var('paged', 0);
		if (empty($page)) {
			$page = get_query_var('page', 0);
		}
		if (empty($page)) {
			$page = 1;
		}
		if (0 < $limit) {
			$pages = ceil($this->total / $limit);
		} else {
			$pages = 1;
		}

		if (!empty($pages) && $page > $pages) {
			$page = $pages;
		}
		if (1 < $page) {
			$s .= " OFFSET " . ($page - 1) * $limit;
		}
		$this->limit = (int)$limit;
		$this->page = (int)$page;
		$this->pages = (int)$pages;
		return $s;
	}

	protected function getRowsWithAttr(&$attr, &$args)
	{
		global $wpdb;

		if (null === ($class = $this->valueForKey($args, 'class'))) {
			return null;
		}
		if (null === ($where = $this->valueForKey($args, 'where'))) {
			$where = $this->composeWhereWithAttr($attr);
		}
		$table = "{$wpdb->prefix}cw_{$this->game}s";
		$count = $wpdb->get_var("SELECT count(*) FROM `{$table}`{$where};");
		$this->total = $count;

		if (null === ($limit = $this->valueForKey($args, 'limit'))) {
			$limit = $this->composeLimitWithAttr($attr);
		}
		if (null === ($select = $this->valueForKey($args, 'select'))) {
			return null;
		}
		$order = $this->valueForKey($args, 'order', '');
		$rows = $wpdb->get_results("SELECT {$select} FROM `{$table}`{$where}{$order}{$limit};", ARRAY_A);
		$list = [];
		foreach ($rows as $row) {
			$obj = new $class($row);
			$list[] = $obj;
		}
		$this->found = count($rows);

		return $list;
	}

	/* ---------------------------------------------- */
	/* Render
	/* ---------------------------------------------- */
	public function renderShortCode(&$attr)
	{
		return $this->render($attr);
	}

	public function render($attr)
	{
		$file = CWPATH . "templates/game-{$this->figure}.php";
		if (!file_exists($file)) {
			return "ERROR: File missing!";
		}
		ob_start();
		include($file);
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public function renderPagination()
	{
		$pages = $this->pages;
		if (empty($pages) || 1 >= $pages) {
			return;
		}
		$page = $this->page;
		echo '<div class="cw-w-100 cw-text-center"><div class="cw-inline-block">' . PHP_EOL;
		echo '<ul class="cw-pagination">' . PHP_EOL;
		$url = get_pagenum_link($this->page - 1);
		$disabled = 1 >= $page ? " disabled" : "";
		echo '<li class="cw-page-item"><a class="cw-page-link' . $disabled . '" href="' . $url . '">' . __('Previous') . '</a></li>';
		for ($p = 1; $p <= $pages; $p++) {
			$url = get_pagenum_link($p);
			if ($p == $page) {
				echo '<li class="cw-page-item"><a class="cw-page-link active" href="' . $url . '">' . $p . '</a></li>';
			} else {
				echo '<li class="cw-page-item"><a class="cw-page-link" href="' . $url . '">' . $p . '</a></li>';
			}
		}
		$url = get_pagenum_link($page + 1);
		$disabled = $page >= $pages ? " disabled" : "";
		echo '<li class="cw-page-item"><a class="cw-page-link' . $disabled . '" href="' . $url . '">' . __('Next') . '</a></li>';
		echo '</ul>' . PHP_EOL;
		echo '</div></div>' . PHP_EOL;
	}

	/* ---------------------------------------------- */
	/* Render Table List
	/* ---------------------------------------------- */
	public function countEnabledTableColumns(&$options)
	{
		$column_idx = 0;
		if ($options) {
			foreach ($options as $option) {
				if ($this->valueForKey($option, 'enabled', 0)) {
					$column_idx++;
				}
			}
		}
		return $column_idx;
	}

	public function renderTableEmpty(&$list, &$options)
	{
		if ($empty = empty($list)) {
			$text = esc_html($this->valueForKey($options, 'empty_list', "Empty List!"));
			echo "<tr><th class=\"cw-text-center\">" . $text . "</th></tr>" . PHP_EOL;
		}
		return $empty;
	}

	public function renderTableHeader(&$list, &$options, $nrof_columns)
	{
		if ($this->renderTableEmpty($list, $options)) {
			return;
		}
		if (!isset($options['table_headers']) || !$options['table_headers']) {
			return;
		}
		echo "<tr class=\"cw-tr-seps\">" . PHP_EOL;
		if ($options) {
			foreach ($options as $key => $option) {
				if ($this->valueForKey($option, 'enabled', 0)) {
					if ('pulldown' == $key) {
						$collapse = $this->valueForKey($options, 'drawer');
					} else {
						$collapse = $this->valueForKey($option, 'collapse');
					}
					$classes = ($collapse && 'color' != $key && 'title' != $key) ? " cw-hide-{$collapse}" : "";
					if ('color' == $key) {
						echo "<th class=\"cw-th-seps{$classes}\" style=\"width:5px;padding:0;\"></th>" . PHP_EOL;
					} else {
						$title = esc_html($this->valueForKey($option, 'title', ""));
						echo "<th class=\"cw-th-seps{$classes}\">{$title}</th>" . PHP_EOL;
					}
				}
			}
		}
		echo "</tr>" . PHP_EOL;
	}

	public function renderTableBody(&$list, &$options, $nrof_columns)
	{
		if (!empty($list)) {
			$grouped = $this->valueForKey($options, 'grouped', 0);
			$row_idx = 0;
			$time_change = 0;
			$type_change = -1;
			$group = false;
			foreach ($list as $bg) {
				if ($grouped) {
					if (($type = $bg->type()) !== null) {
						if ($type != $type_change) {
							$type_change = $type;
							$group = true;
						}
						if ($type == CWEnumGameType::Dated && ($start = $bg->localStartDate())) {
							$date = substr($start, 0, 10);
							$time = strtotime($date);
							if ($time != $time_change) {
								$time_change = $time;
								$group = true;
							}
						} else {
							$time = null;
						}
						if ($group) {
							$this->renderTableGroup($nrof_columns, $type, $time);
							$group = false;
						}
					}
				}
				$this->renderTableRow($bg, $options, $row_idx);
				$this->renderTableRowDetails($bg, $options, $row_idx, $nrof_columns);
				$row_idx++;
			}
		}
	}

	public function renderTableGroup($nrof_columns, $v1 = null, $v2 = null)
	{
		if ('results' == $this->figure || 2 == $v1) {
			$format = get_option('date_format');
			$v = date_i18n($format, $v2);
		} else if (1 == $v1) {
			$v = __("Ring", CWSLUG);
		} else if (0 == $v1) {
			$v = __("Sit&Go", CWSLUG);;
		} else {
			$v = "";
		}
		echo "<tr class=\"\"><td class=\"cw-td-seps-group\" colspan=\"{$nrof_columns}\">{$v}</td></tr>" . PHP_EOL;
	}

	public function renderTableRow(&$bg, &$options, $row_idx)
	{
		// Render row with columns
		echo "<tr id=\"cw_tr_{$row_idx}\" class=\"cw-tr-seps\">" . PHP_EOL;
		if ($options) {
			foreach ($options as $key => $option) {
				if ($this->valueForKey($option, 'enabled', 0)) {
					if (('more' == $key || 'title' == $key) && $this->valueForKey($options, 'details_page_id')) {
						$option['page_id'] = $options['details_page_id'];
						if ('more' == $key) {
							$key = 'more.flat';
						}
						$collapse = null;
					} else if ('pulldown' == $key) {
						$collapse = $this->valueForKey($options, 'drawer');
					} else {
						$collapse = $this->valueForKey($option, 'collapse');
					}
					if ($collapse && 'color' != $key && 'title' != $key) {
						$classes = " cw-hide-{$collapse}";
					} else if ('title' == $key) {
						$classes = " cw-td-seps-title";
					} else {
						$classes = "";
					}
					echo $this->renderTableCell($bg, $key, $option, $row_idx, $classes) . PHP_EOL;
				}
			}
		}
		echo "</tr>" . PHP_EOL;
	}

	public function renderTableCell(&$bg, $key, $option, $row_idx, $classes)
	{
		if (($v = $bg->formattedParamValue($key, $option)) !== null) {
			if ('pulldown' == $key) {
				$src = CWURL . "imgs/expand-32@3x.png";
				$v = "<span class=\"cw-pulldown\" id=\"cw_tr_{$this->id}_{$row_idx}\"><img src=\"{$src}\"></span>";
			}
			if ('color' == $key) {
				$s = "<td class=\"cw-td-seps{$classes}\" style=\"width:5px;padding:0;background-color:{$v};\"></td>";
			} else {
				$w = (int)$this->valueForKey($option, 'width', 0);
				$styles = (0 < $w) ? " style=\"width:{$w}px;white-space:nowrap;\"" : "";
				$classes .= " cw-text-" . $this->valueForKey($option, 'align', 'center');
				$s = "<td class=\"cw-td-seps{$classes}\"{$styles}>{$v}</td>";
			}
		} elseif ('color' == $key) {
			// We have to set the style for the color even if we do not have a value
			$s = "<td class=\"cw-td-seps\" style=\"width:5px;padding:0;\"></td>";
		} else {
			// No value, empty cell
			$w = (int)$this->valueForKey($option, 'width', 0);
			$styles = (0 < $w) ? " style=\"width:{$w}px;white-space:nowrap;\"" : "";
			$s = "<td class=\"cw-td-seps{$classes}\"{$styles}></td>";
		}
		return $s;
	}

	public function renderTableRowDetails(&$bg, &$options, $row_idx, $nrof_columns)
	{
		// Create details
		$drawer_open = $this->valueForKey($options, 'drawer');
		$classes = $drawer_open ? " cw-show-{$drawer_open}" : "";
		echo "<tr class=\"cw-tr-seps-drawer\"><td class=\"cw-td-seps-drawer\" colspan=\"{$nrof_columns}\">" . PHP_EOL;
		echo "<div id=\"cw_tr_details_{$this->id}_{$row_idx}\" class=\"cw-tr-seps-details{$classes}\">" . PHP_EOL;
		// Title
		if (isset($options['title']['detail']) && $options['title']['detail']) {
			$value = $bg->formattedParamValue('title');
			if (!empty($value)) {
				echo "<h4>{$value}</h4>" . PHP_EOL;
			}
		}
		// Short desc
		if (isset($options['shortdesc']['detail']) && $options['shortdesc']['detail']) {
			$value = $bg->formattedParamValue('shortdesc');
			if (!empty($value)) {
				echo "<span class=\"cw-font-size-16\">{$value}</span>" . PHP_EOL;
			}
		}
		// Additional Buy-Ins  (Re-Entry, Re-Buy, etc)
		if (isset($options['additional_buyins']['detail']) && $options['additional_buyins']['detail']) {
			$value = $bg->formattedParamValue('additional_buyins');
			if (!empty($value)) {
				echo "<p>{$value}</p>" . PHP_EOL;
			}
		}

		echo "<div class=\"cw-row\">" . PHP_EOL;
		$show_levels = isset($options['levels']['detail']) && $options['levels']['detail'] && !empty($bg->levels);

		// Left side
		echo "<!-- Details -->" . PHP_EOL;
		echo "<div class=\"cw-col-12 cw-col-lg-6 cw-mb-1\">" . PHP_EOL;
		echo "<table class=\"cw-table-details\">" . PHP_EOL;
		$detail_count = 0;
		foreach ($options as $key => $option) {
			if ('array' == gettype($option)) {
				if ('title' != $key && 'shortdesc' != $key && 'more' != $key && 'color' != $key && 'additional_buyins' != $key && 'levels' != $key) {
					if ($this->valueForKey($option, 'detail')) {
						$l = $this->valueForKey($option, 'title', "");
						if ('reg' == $key) $key = 'reg.level';
						if ('counttables' == $key && 'cashgame' == $this->game) $key = 'counttablenames';
						if (null !== ($v = $bg->formattedParamValue($key))) {
							$s = "<tr><td class=\"cw-label cw-text-start\">{$l}:&nbsp;&nbsp;</td><td class=\"cw-cell cw-text-start\">{$v}</td></tr>";
							echo $s . PHP_EOL;
							$detail_count++;
						}
					}
				}
			}
		}
		echo "</table>" . PHP_EOL;
		// Link to details
		if ($details_page_id = $this->valueForKey($options, 'details_page_id')) {
			if (isset($options['more']['detail']) && $options['more']['detail']) {
				$option = $options['more'];
				$option['page_id'] = $details_page_id;
				if (($v = $bg->formattedParamValue('more', $option)) !== null) {
					echo $v . PHP_EOL;
					// echo "<div class=\"cw-text-center\">$v</div>" . PHP_EOL;
				}
			}
		}
		echo "</div>" . PHP_EOL;
		echo "<!-- End Details -->" . PHP_EOL;
		// Left side End

		// Right side
		// Levels
		if ($show_levels) {
			echo "<div class=\"cw-col-12 cw-col-lg-6 cw-mb-1\">" . PHP_EOL;
			$bg->renderTableLevels();
			echo "</div>" . PHP_EOL;
		}

		echo "</div>" . PHP_EOL;
		echo "</div>" . PHP_EOL; // End Details

		echo "</td></tr>" . PHP_EOL;
	}
}
