<?php
defined('ABSPATH') or die('No direct access');
class CWCashGameList extends CWBaseList
{
	protected $games = null;

	public function __construct($figure, $id = 0)
	{
		parent::__construct('cashgame', $figure, $id);
	}

	/* ---------------------------------------------- */
	/* Override
	/* ---------------------------------------------- */
	public function loadData($attr)
	{
		if (0 == $this->generalOptionForKey($this->game . 's_manager', 0)) {
			return null;
		}
		if (!$this->list) {
			switch ($this->valueForKey($attr, 'type', CWEnumGameType::Ring)) {
				case CWEnumGameType::SitGo: // Sit&Go
					$order = " ORDER BY `title`";
					break;
				case CWEnumGameType::Dated: // Dated
					$order = " ORDER BY `start_local`, `title`";
					break;
				default:
					$order = " ORDER BY `title`";
			}
			$args = [
				'class' => 'CWCashGame',
				'select' => '`ID`,`params`, `tables`',
				'order' => $order
			];
			$list = $this->getRowsWithAttr($attr, $args);
			usort($list, [$this, 'gameOrder']);
			$this->list = $list;
		}
		return $this->list;
	}

	/* ---------------------------------------------- */
	/* Functions
	/* ---------------------------------------------- */
	protected function gameOrder($a, $b)
	{
		$v1 = $a->game();
		$v2 = $b->game();
		if ($v1 == $v2) {
			$v1 = $a->sort();
			$v2 = $b->sort();
			if ($v1 == $v2) {
				$v1 = $a->paramValue('csb', 0);
				$v2 = $b->paramValue('csb', 0);
				if ($v1 == $v2) {
					$v1 = $a->paramValue('bsb', 0);
					$v2 = $b->paramValue('bsb', 0);
					if ($v1 == $v2) {
						$v1 = $a->title();
						$v2 = $b->title();
						if ($v1 == $v2) {
							return 0;
						}
					}
				}
			}
		}
		return ($v1 <=> $v2);
	}
	/* ---------------------------------------------- */
	/* Lists
	/* ---------------------------------------------- */
	public function listOfGames()
	{
		if ($this->games === null) {
			$list = [];
			$list[0] = ['name' => "Texas Hold'em", "short" => "HE"];
			$list[1] = ['name' => "Omaha", "short" => "OM"];
			$list[2] = ['name' => "Omaha Eight or Better", "short" => "OM/8B"];
			$list[3] = ['name' => "7 Card Stud", "short" => "Stud"];
			$list[4] = ['name' => "7 Card Stud Eight or Better", "short" => "Stud/8B"];
			$list[5] = ['name' => "RAZZ", "short" => "RAZZ"];
			$list[6] = ['name' => "5 Card Draw", "short" => "Draw"];
			$list[7] = ['name' => "Single Draw 2-7 Lowball", "short" => "SD/27L"];
			$list[8] = ['name' => "Triple Draw 2-7 Lowball", "short" => "TD/27L"];
			$list[9] = ['name' => "Triple Draw A-5 Lowball", "short" => "TD/A5L"];
			$list[20] = ['name' => "HORSE", "short" => "HORSE"];
			$list[21] = ['name' => "Short-Deck Hold'em", "short" => "SHE"];
			$list[10] = ['name' => "Badugi", "short" => "Badugi"];
			$list[11] = ['name' => "Crazy Pineapple", "short" => "CP"];
			$list[13] = ['name' => "Chinese Poker", "short" => "ChP"];
			$list[14] = ['name' => "Open-Face Chinese Poker", "short" => "OFC"];
			$list[15] = ['name' => "Round of Each", "short" => "RoE"];
			$list[16] = ['name' => "Hold'em X", "short" => "HX"];
			$list[18] = ['name' => "Easy Hold'em", "short" => "EH"];
			$list[19] = ['name' => "3Card Poker", "short" => "3Card"];
			$list[17] = ['name' => "Aces", "short" => "Aces"];
			$list[22] = ['name' => "Mixed", "short" => "MIXED"];
			$list[12] = ['name' => "Dealers Choice", "short" => "DC"];
			$this->games = $list;
		}
		return $this->games;
	}

	/* ---------------------------------------------- */
	/* Render Table List
	/* ---------------------------------------------- */
	public function renderTableBody(&$list, &$options, $nrof_columns)
	{
		if (!empty($list)) {
			$grouped = $this->valueForKey($options, 'grouped', 0);
			$row_idx = 0;
			$game_change = -1;
			$group = false;
			foreach ($list as $bg) {
				if ($grouped) {
					if (($game = $bg->game()) !== null) {
						if ($game != $game_change) {
							$game_change = $game;
							$group = true;
						}
						if ($group) {
							$this->renderTableGroup($nrof_columns, $game);
							$group = false;
						}
					}
				}
				$this->renderTableRow($bg, $options, $row_idx);
				$this->renderTableRowDetails($bg, $options, $row_idx, $nrof_columns);
				$row_idx++;
			}
		}
	}

	public function renderTableGroup($nrof_columns, $game = null, $v2 = null)
	{
		$games = $this->listOfGames();
		if ($game >= 0 && $game < count($games)) {
			$v = $games[$game]['name'];
		} else {
			$v = "";
		}
		echo "<tr class=\"\"><td class=\"cw-td-seps-group\" colspan=\"{$nrof_columns}\">{$v}</td></tr>" . PHP_EOL;
	}
}
