<?php
defined('ABSPATH') or die('No direct access');
class CWGamesTable extends CWListTable
{
   public function prepare_items()
   {
      parent::prepare_items();
      if (isset($_GET['status'])) {
         $v = $_GET['status'];
         if ($v > 0 && $v <= 32 && $v !== '') {
            $this->filter_fields['status'] = (int)$v;
         }
      }
      if (isset($_GET['type'])) {
         $v = $_GET['type'];
         if ($v >= 0 && $v <= 2 && $v !== '') {
            $this->filter_fields['type'] = (int)$v;
         }
      }
      if (isset($_GET['evid'])) {
         $v = $_GET['evid'];
         if ($v >= 0 && $v !== '') {
            $this->filter_fields['EVID'] = (int)$v;
         }
      }
   }

   public function get_bulk_actions()
   {
      $functions = [
         'cw_action_publish_once' => esc_html__('Publish'),
         'cw_action_unpublish' => esc_html__('Unpublish', CWSLUG),
         'cw_control_finish' => esc_html__('Finish', CWSLUG),
         'cw_action_delete' => esc_html__('Delete')
      ];
      return $functions;
   }

   public function handle_row_actions($item, $column_name, $primary)
   {
      if ($primary !== $column_name) {
         return '';
      }
      $status = $item->status(true);
      $pub = $item->paramValue('pub', 0);

      $actions = [];
      if ($pub) {
         $actions['unpublish'] = '<a cw-id="' . $item->id() . '" cw-action="cw_action_unpublish" class="cw-row-action">' . esc_html__('Unpublish', CWSLUG) . '</a>';
      } else {
         $actions['publish_once'] = '<a cw-id="' . $item->id() . '" cw-action="cw_action_publish_once" class="cw-row-action">' . esc_html__('Publish') . '</a>';
      }
      if (CWEnumGameStatus::Seating == $status) {
         $actions['start'] = '<a cw-id="' . $item->id() . '" cw-action="cw_control_start" class="cw-row-action">' . esc_html__('Start', CWSLUG) . '</a>';
      }
      if (CWEnumGameStatus::Running == $status) {
         $actions['finish'] = '<a cw-id="' . $item->id() . '" cw-action="cw_control_finish" class="cw-row-action">' . esc_html__('Finish', CWSLUG) . '</a>';
      }
      if (CWEnumGameStatus::Finished == $status) {
         $actions['edit'] = '<a cw-id="' . $item->id() . '" cw-action="cw_action_edit" class="cw-row-action">' . esc_html__('Edit', CWSLUG) . '</a>';
      }
      // if (CWEnumGameStatus::Initiate == $status || CWEnumGameStatus::Finished == $status) {
      //    $actions['delete'] = '<a cw-id="' . $item->id() . '" cw-action="cw_action_delete" class="cw-row-action">' . esc_html__('Delete') . '</a>';
      // }
      return $this->row_actions($actions);
   }

   public function get_columns()
   {
      $columns = [
         'cb' => '<input type="checkbox" />',
         'cw-title' => esc_html__('Title', CWSLUG),
         'cw-status' => esc_html__('Status', CWSLUG),
         'cw-start_local' => esc_html__('Start', CWSLUG),
         'cw-pub' => esc_html__('Published'),
         'cw-lcid' => esc_html__('LCID', CWSLUG),
         'cw-evid' => esc_html__('EVID', CWSLUG),
         'cw-id' => esc_html__('ID', CWSLUG)
      ];
      return $columns;
   }

   public function get_hidden_columns()
   {
      $columns = [];
      return $columns;
   }

   public function get_sortable_columns()
   {
      return [
         'cw-title' => ['title', 'asc'],
         'cw-status' => ['status', 'desc'],
         'cw-lcid' => ['LCID', 'asc'],
         'cw-evid' => ['EVID', 'asc'],
         'cw-start_local' => ['start_local', 'asc'],
         'cw-id' => ['ID', 'desc']
      ];
   }

   public function column_default($item, $column_name)
   {
      if (3 >= strlen($column_name)) {
         return null;
      }
      $column_name = substr($column_name, 3);
      switch ($column_name) {
         case 'id':
            $v = $item->id();
            break;
         case 'title':
            $color = $item->formattedParamValue('color',);
            if (empty($color)) {
               $color = 'rgba(255,255,255,0)';
            }
            $title = $item->formattedParamValue($column_name);
            $v = '<div class="cw-bullet" style="background-color:' . $color . ';"></div>&nbsp;&nbsp;<span class="row-title">' . $title . '</span>';
            break;
         case 'lcid':
            $v = $item->lcid();
            break;
         case 'ttid':
         case 'cgid':
            $v = $item->formattedParamValue($this->idfield);
            break;
         case 'evid':
            $v = $item->evid();
            break;
         case 'start_local':
            $v = $item->formattedParamValue('start_date_time_local');
            break;
         case 'status':
            $v = $item->formattedParamValue('status.img');
            break;
         case 'pub':
            $v = $item->formattedParamValue('pub');
            break;
         default:
            $v =  "?";
            break;
      }
      return $v;
   }

   /* ---------------------------------------------- */
   /* Extra Table Nav */
   /* ---------------------------------------------- */
   protected function tableNavStatusDropdown()
   {
      $status = isset($_GET['status']) ? $_GET['status'] : '';
      $list = [];
      $list[''] = 'All states';
      $names = CWBaseGame::listOfStatusNames();
      foreach ($names as $value => $name) {
         if ($value <= 16) {
            $list[$value] = $name;
         }
      }
      $v = '<label for="filter-status" class="screen-reader-text">States</label>' . PHP_EOL;
      $v .= '<select name="status" id="filter-status">' . PHP_EOL;
      foreach ($list as $value => $name) {
         $option = sprintf("<option %s value='%s'>%s</option>\n", selected($status, $value, false), esc_attr($value), esc_html__($name, CWSLUG));
         $v .= $option;
      }
      $v .= '</select>' . PHP_EOL;
      return $v;
   }

   protected function tableNavTypeDropdown()
   {
      $type = isset($_GET['type']) ? $_GET['type'] : '';
      $list = ['' => 'All types', 0 => 'Sit&Go', 1 => 'Ring', 2 => 'Dated'];
      $v = '<label for="filter-type" class="screen-reader-text">Types</label>' . PHP_EOL;
      $v .= '<select name="type" id="filter-type">' . PHP_EOL;
      foreach ($list as $value => $name) {
         $option = sprintf("<option %s value='%s'>%s</option>\n", selected($type, $value, false), esc_attr($value), esc_html__($name, CWSLUG));
         $v .= $option;
      }
      $v .= '</select>' . PHP_EOL;
      return $v;
   }

   protected function tableNavEVIDField()
   {
      $evid = isset($_GET['evid']) ? $_GET['evid'] : "";
      $v = '<label for="filter-evid" class="screen-reader-text">Event ID</label>' . PHP_EOL;
      $v .= '<input type="text" name="evid" id="filter-evid" placeholder="' . esc_html__('EVID', CWSLUG) . '" value="' . $evid . '" size="4"/>' . PHP_EOL;
      return $v;
   }

   protected function extra_tablenav($which)
   {
      echo '<div class="alignleft actions">';
      if ('top' === $which) {
         $output = $this->tableNavStatusDropdown();
         $output .= $this->tableNavTypeDropdown();
         $output .= $this->tableNavLCIDField();
         $output .= $this->tableNavEVIDField();
         if (!empty($output)) {
            echo $output;
            submit_button(__('Filter'), '', 'filter_action', false, array('id' => 'post-query-submit'));
         }
      }
      echo '</div>' . PHP_EOL;
   }

   /* ---------------------------------------------- */
   /* Ajax */
   /* ---------------------------------------------- */
   public function ajaxControl(int $status)
   {
      global $wpdb;

      $nonce = $_REQUEST['_wpnonce'];
      $page = $_REQUEST['page'];
      $tablename = $this->tablename;
      $ids = $_REQUEST['cw_ids'];
      if (empty($nonce) || empty($page) || empty($tablename) || empty($ids)) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'data', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      $action_toplevel = "bulk-toplevel_page_" . $page;
      $action_casinoware = "bulk-" . CWSLUG . "_page_" . $page;
      if (!wp_verify_nonce($nonce, $action_toplevel) && !wp_verify_nonce($nonce, $action_casinoware)) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'nonce', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      $table = $wpdb->prefix . 'cw_' . $tablename;

      if (!$this->startTransaction()) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'start', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      foreach ($ids as $id) {
         $q = $wpdb->prepare('SELECT `status`,`params` FROM `' . $table . '` WHERE `ID` = %d;', $id);
         $data = $wpdb->get_row($q, ARRAY_A);
         if (16 == $status && 8 == $data['status']) {
            if (!isset($data['params'])) {
               $this->rollbackTransaction();
               $error_msg = __("Value 'params' missing.");
               $res = ['result' => 'update', 'errmsg' => $error_msg];
               wp_send_json_error($res);
               wp_die();
               return;
            }
            $params = json_decode($data['params'], true, 6);
            $params['status'] = $status;
            $data['params'] = json_encode($params);
            $data['status'] = $status;

            $r = $wpdb->update($table, $data, ['ID' => $id], ['%d', '%s'], ['%d']);
            if (!$r && !empty($wpdb->last_error)) {
               $this->rollbackTransaction();
               $error_msg = $wpdb->last_error;
               $res = ['result' => 'update', 'errmsg' => $error_msg];
               wp_send_json_error($res);
               wp_die();
            }
         }
      }

      if (!$this->commitTransaction()) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'commit', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      wp_send_json_success();
      wp_die();
   }

   public function ajaxPublish(int $value)
   {
      global $wpdb;

      $nonce = $_REQUEST['_wpnonce'];
      $page = $_REQUEST['page'];
      $tablename = $this->tablename;
      $ids = $_REQUEST['cw_ids'];
      if (empty($nonce) || empty($page) || empty($tablename) || empty($ids)) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'data', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      $action_toplevel = "bulk-toplevel_page_" . $page;
      $action_casinoware = "bulk-" . CWSLUG . "_page_" . $page;
      if (!wp_verify_nonce($nonce, $action_toplevel) && !wp_verify_nonce($nonce, $action_casinoware)) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'nonce', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      $table = $wpdb->prefix . 'cw_' . $tablename;

      if (!$this->startTransaction()) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'start', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      foreach ($ids as $id) {
         $q = $wpdb->prepare('SELECT `pub`,`params` FROM `' . $table . '` WHERE `ID` = %d;', $id);
         $data = $wpdb->get_row($q, ARRAY_A);
         if (!isset($data['params'])) {
            $this->rollbackTransaction();
            $error_msg = __("Value 'params' missing.");
            $res = ['result' => 'update', 'errmsg' => $error_msg];
            wp_send_json_error($res);
            wp_die();
            return;
         }
         $params = json_decode($data['params'], true, 6);
         $params['pub'] = $value;
         $data['params'] = json_encode($params);
         $data['pub'] = $value;

         $r = $wpdb->update($table, $data, ['ID' => $id], ['%d', '%s'], ['%d']);
         if (!$r && !empty($wpdb->last_error)) {
            $this->rollbackTransaction();
            $error_msg = $wpdb->last_error;
            $res = ['result' => 'update', 'errmsg' => $error_msg];
            wp_send_json_error($res);
            wp_die();
         }
      }

      if (!$this->commitTransaction()) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'commit', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      wp_send_json_success();
      wp_die();
   }

   public function ajaxEdit()
   {
      global $wpdb;

      $nonce = $_REQUEST['_wpnonce'];
      $page = $_REQUEST['page'];
      $tablename = $this->tablename;
      $ids = $_REQUEST['cw_ids'];
      if (empty($nonce) || empty($page) || empty($tablename) || empty($ids)) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'data', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }
      wp_send_json_success();
      wp_die();
   }
}
