<?php
defined('ABSPATH') or die('No direct access');
class CWListTable extends WP_List_Table
{
   use CWFoundationTrait;

   // Table
   public $tablename;
   public $idfield;
   public $fieldnames;
   public $fieldtypes;
   public $fielddefaults;

   // List
   public $total = 0;
   public $limit = 20;

   // Serach and order
   protected $search_fields = []; // all fields concatinataed with OR, value is $search_value
   protected $search_value = null; // used for $search_fields
   protected $filter_fields = []; // single fields for equal operation, like LCID
   protected $paged = null;
   protected $order_by = null;
   protected $order = null;

   // Transactions
   protected $transactions = 0;

   // Notes
   protected $notes = [];

   public function __construct($tablename, $idfield, $fieldnames, $fieldtypes, $fielddefaults = null)
   {
      if (count($fieldnames) != count($fieldtypes)) {
         $nrof_name = count($fieldnames);
         $nrof_types = count($fieldtypes);
         $class = __CLASS__;
         throw new Exception("{$class} has different number of field names and typs {$nrof_name} != {$nrof_types}!");
      }
      parent::__construct();
      $this->tablename = $tablename;
      $this->idfield = $idfield;
      $this->fieldnames = $fieldnames;
      $this->fieldtypes = $fieldtypes;
      $this->fielddefaults = $fielddefaults;
   }

   /* ---------------------------------------------- */
   /* Notes & Errors */
   /* ---------------------------------------------- */
   public function printNotes()
   {
      foreach ($this->notes as $note) {
         $type = $this->valueForKey($note, 'type', 'error');
         $msg = $this->valueForKey($note, 'msg', '?');
         echo '<div class="' . $type . ' notice">' . $msg . '</div>' . PHP_EOL;
      }
   }

   protected function addNote($text, $type = 'error') // updated, pdate-nag
   {
      $note['msg'] = $text;
      $note['type'] = $type;
      $this->notes[] = $note;
   }

   /* ---------------------------------------------- */
   /* Transactions */
   /* ---------------------------------------------- */
   public function startTransaction()
   {
      global $wpdb;
      if ($this->transactions > 0) {
         $this->transactions++;
         return true;
      }

      $r = $wpdb->query("START TRANSACTION;") < 0 ? false : true;
      $this->transactions = 1;
      return $r;
   }

   public function commitTransaction()
   {
      global $wpdb;
      if ($this->transactions > 1) {
         $this->transactions--;
         return true;
      }

      $r = $wpdb->query("COMMIT;") < 0 ? false : true;
      $this->transactions = 0;
      return $r;
   }

   public function rollbackTransaction()
   {
      global $wpdb;
      if ($this->transactions > 1) {
         $this->transactions--;
         return true;
      }
      $r = $wpdb->query("ROLLBACK;") < 0 ? false : true;
      $this->transactions = 0;
      return $r;
   }

   /* ---------------------------------------------- */
   /* Functions */
   /* ---------------------------------------------- */

   /* ---------------------------------------------- */
   /* List Table */
   /* ---------------------------------------------- */
   public function register()
   {
      $values = ['ajaxurl' => admin_url('admin-ajax.php')];
      wp_localize_script(CWSLUG, 'cw', $values);
   }

   public function prepare_items()
   {
      $this->order_by = isset($_GET['orderby']) ? $_GET['orderby'] : '';
      $this->order = isset($_GET['order']) ? $_GET['order'] : '';
      $this->search_value = isset($_GET['s']) ? $_GET['s'] : '';
      $this->paged = isset($_GET['paged']) ? (int)$_GET['paged'] : 1;
      if (isset($_GET['lcid'])) {
         $v = $_GET['lcid'];
         if ($v >= 0 && $v !== '') {
            $this->filter_fields['LCID'] = (int)$v;
         }
      }

      $columns = $this->get_columns();
      $columns_hidden = $this->get_hidden_columns();
      $columns_sortable = $this->get_sortable_columns();
      $this->_column_headers = [$columns, $columns_hidden, $columns_sortable];
   }

   public function getRows($select, $class)
   {
      global $wpdb;

      $where = "";
      // Prepare filter fields
      if (!empty($this->filter_fields)) {
         $i = 0;
         foreach ($this->filter_fields as $field => $value) {
            if (false !== ($idx = array_search($field, $this->fieldnames))) {
               if (0 < $i) {
                  $where .= ' AND ';
               }
               $type = $this->fieldtypes[$idx];
               if ('%s' == $type) {
                  $v = addslashes($value);
                  $where .=  "`{$field}` LIKE '%{$v}%'";
               } else {
                  $where .=  "`{$field}` = " . sprintf($type, $value);
               }
               $i++;
            }
         }
      }

      // Prepare search fields
      if (!empty($this->search_fields) && !empty($this->search_value)) {
         if ($and = (0 < strlen($where))) {
            $where .= ' AND (';
         }
         $value = addslashes($this->search_value);
         $i = 0;
         foreach ($this->search_fields as $field) {
            if (false !== ($idx = array_search($field, $this->fieldnames))) {
               if (0 < $i) {
                  $where .= ' OR ';
               }
               $type = $this->fieldtypes[$idx];
               if ('%s' == $type) {
                  $where .=  "`{$field}` LIKE '%{$value}%'";
               } else {
                  $where .=  "`{$field}` = " . sprintf($type, $value);
               }
               $i++;
            }
         }
         if ($and) {
            $where .= ')';
         }
      }
      if (strlen($where) > 0) {
         $where = " WHERE " . $where;
      }

      // Prepare order
      if (!empty($this->order_by)) {
         $orderby = " ORDER BY ";
         if ($this->order_by == 'status') {
            $orderby .= '(`status` & 31)';
         } else {
            $orderby .= '`' . $this->order_by . '`';
         }
         if (!empty($this->order)) {
            $orderby .= " " . $this->order;
         }
      } else {
         $orderby = "";
      }

      // Prepare limit and offset
      $limits = " LIMIT {$this->limit}";
      if ($this->paged > 1) {
         $offset = ($this->paged - 1) * $this->limit;
         $limits .= " OFFSET " . $offset;
      }

      // Query
      $count = $wpdb->get_var("SELECT count(*) AS count FROM `{$wpdb->prefix}cw_{$this->tablename}`{$where}");
      if (null === $count) {
         $this->addNote(__("Error") . " :" . $wpdb->last_error);
         return false;
      } elseif ($count > 0) {
         $rows = $wpdb->get_results("SELECT {$select} FROM `{$wpdb->prefix}cw_{$this->tablename}`{$where}{$orderby}{$limits};", ARRAY_A);
         if (false === $rows) {
            $this->addNote(__("Error") . " :" . $wpdb->last_error);
            return false;
         }
         $items = [];
         foreach ($rows as $tt) {
            $items[] = new $class($tt);
         }
         $this->total = $count;
         $this->items = $items;
      } else {
         $this->total = 0;
         $this->items = null;
      }

      return true;
   }

   public function column_cb($item)
   {
      if ($item) {
         $checkbox = '<input type="checkbox" name="cw_ids[]" value="' . $item->id() . '"/>';
      } else {
         $checkbox = '';
      }
      return $checkbox;
   }

   /* ---------------------------------------------- */
   /* Extra Table Nav */
   /* ---------------------------------------------- */
   protected function tableNavLCIDField()
   {
      $lcid = isset($_GET['lcid']) ? $_GET['lcid'] : "";
      $v = '<label for="filter-lcid" class="screen-reader-text">Location ID</label>' . PHP_EOL;
      $v .= '<input type="text" name="lcid" id="filter-lcid" placeholder="' . __('LCID', CWSLUG) . '" value="' . $lcid . '" size="4"/>' . PHP_EOL;
      return $v;
   }

   /* ---------------------------------------------- */
   /* ajax Functions */
   /* ---------------------------------------------- */
   protected function nonceFromRequest()
   {
      return $_REQUEST['_wpnonce'];
   }

   protected function idsFromRequest()
   {
      return $_REQUEST['cw_ids'];
   }

   public function ajaxDelete()
   {
      global $wpdb;

      $tablename = $this->tablename;
      $nonce = $this->nonceFromRequest();
      if (empty($nonce) || empty($tablename)) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'data', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }
      $ids = $this->idsFromRequest();
      if (empty($ids)) {
         $error_msg = __('No rows selected.', CWSLUG);
         $res = ['result' => 'table', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      $table = $wpdb->prefix . 'cw_' . $tablename;
      $in = implode(',', $ids);
      $q = "DELETE FROM {$table} WHERE ID IN ({$in});";
      if (!$wpdb->query($q)) {
         $error_msg = $wpdb->last_error;
         $res = ['result' => 'delete', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      wp_send_json_success();
      wp_die();
   }

   public function ajaxTruncate()
   {
      global $wpdb;

      $tablename = $this->tablename;
      $nonce = $this->nonceFromRequest();
      if (empty($nonce) || empty($tablename)) {
         $error_msg = __('Unable to submit this form, please refresh and try again.');
         $res = ['result' => 'data', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      $table = $wpdb->prefix . 'cw_' . $tablename;
      if (!$wpdb->query("TRUNCATE TABLE {$table};")) {
         $error_msg = $wpdb->last_error;
         $res = ['result' => 'truncate', 'errmsg' => $error_msg];
         wp_send_json_error($res);
         wp_die();
      }

      wp_send_json_success();
      wp_die();
   }

   public function ajaxRecreate()
   {
      global $wpdb;

      $manager = new CWPlugInManager();
      $errors = $manager->recreateTable($this->tablename);
      if (0 < $errors) {
			$error_msg = __('Error: '.$wpdb->last_error);
			$res = ['result' => 'table', 'errmsg' => $error_msg];
			wp_send_json_error($res);
         wp_die();
		}

      wp_send_json_success();
      wp_die();
   }
}
