<?php
defined('ABSPATH') or die('No direct access');
class CWPlayersTable extends CWListTable
{
   public function __construct()
   {
      $idfield = 'PLID';
		$fieldnames = [
			'LCID', 'modified_utc', 'modified_local',
			$idfield, 'address', 'ensign', 'anonymous', 'MEID', 'dname', 'fname', 'lname', 'nname', 'email', 'mdno', 'fpp',
			'sctime', 'sctables', 'tctime', 'tctables',
			'bal0value', 'bal1value', 'bal2value', 'bal3value', 'bal4value',
			'stat0value', 'stat0count', 'stat0ptime', 'stat1value', 'stat1count', 'stat1ptime', 'stat2value', 'stat2count', 'stat2ptime', 'stat3value', 'stat3count', 'stat3ptime', 'stat4value', 'stat4count', 'stat4ptime',
			'earn0value', 'earn0count', 'earn0ptime', 'earn1value', 'earn1count', 'earn1ptime', 'earn2value', 'earn2count', 'earn3ptime', 'earn3value', 'earn3count', 'earn3ptime', 'earn4value', 'earn4count', 'earn4ptime'
		];
		$fieldtypes = [
			'%d', '%s', '%s',
			'%d', '%d', '%s', '%d', '%d', '%s', '%s', '%s', '%s', '%s', '%s', '%d',
			'%d', '%d', '%d', '%d',
			'%f', '%f', '%f', '%f', '%f',
			'%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d',
			'%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d', '%f', '%d', '%d'
		];
      parent::__construct('players', $idfield, $fieldnames, $fieldtypes);
   }

   public function prepare_items()
   {
      parent::prepare_items();
      $this->search_fields = ['dname', 'email'];
      $this->getRows('*', 'CWPlayer');

      $args = ['total_items' => $this->total, 'per_page' => $this->limit];
      $this->set_pagination_args($args);
   }

   public function get_bulk_actions()
   {
      $functions = [
         'cw_action_delete' => esc_html__('Delete')
      ];
      return $functions;
   }
   
   public function get_columns()
   {
      $currencies = $this->allCurrencies();
      $columns = [
         'cb' => '<input type="checkbox" />',
         'cw-ensign.dname' => __('Name', CWSLUG),
         'cw-email' => __('Email', CWSLUG),
         'cw-stat0' => __('Season', CWSLUG) . ' ' . $currencies[0],
         'cw-stat1' => __('Season', CWSLUG) . ' ' . $currencies[1],
         'cw-stat2' => __('Season', CWSLUG) . ' ' . $currencies[2],
         'cw-stat3' => __('Season', CWSLUG) . ' ' . $currencies[3],
         'cw-stat4' => __('Season', CWSLUG) . ' ' . $currencies[4],
         'cw-earn0' => __('Total', CWSLUG) . ' ' . $currencies[0],
         // 'cw-earn1' => __('Total', CWSLUG) . ' ' . $currencies[1],
         // 'cw-earn2' => __('Total', CWSLUG) . ' ' . $currencies[2],
         'cw-lcid' => __('LCID', CWSLUG),
         'cw-plid' => __('PLID', CWSLUG),
         'cw-id' => __('ID', CWSLUG)
      ];
      return $columns;
   }

   public function get_hidden_columns()
   {
      $columns = [];
      return $columns;
   }

   public function get_sortable_columns()
   {
      return [
         'cw-ensign.dname' => ['dname', 'asc'],
         'cw-email' => ['email', 'asc'],
         'cw-stat0' => ['stat0value', 'desc'],
         'cw-stat1' => ['stat1value', 'desc'],
         'cw-stat2' => ['stat2value', 'desc'],
         'cw-stat3' => ['stat3value', 'desc'],
         'cw-stat4' => ['stat4value', 'desc'],
         'cw-earn0' => ['earn0value', 'desc'],
         'cw-earn1' => ['earn1value', 'desc'],
         'cw-earn2' => ['earn2value', 'desc'],
         'cw-earn3' => ['earn3value', 'desc'],
         'cw-earn4' => ['earn4value', 'desc'],
         'cw-lcid' => ['LCID', 'asc'],
         'cw-plid' => ['PLID', 'asc'],
         'cw-id' => ['ID', 'desc']
      ];
   }

   public function column_default($item, $column_name)
   {
      if (3 >= strlen($column_name)) {
         return null;
      }
      $column_name = substr($column_name, 3);
      return $item->formattedValue($column_name);
   }

//    protected function get_views() { 
//       $status_links = array(
//           "all"       => __("<a href='#'>All</a>"),
//           "published" => __("<a href='#'>Published</a>"),
//           "trashed"   => __("<a href='#'>Trashed</a>")
//       );
//       return $status_links;
//   }
   // public function get_bulk_actions()
   // {
   //    $functions = [
   //       'cw_action_delete' => __('Delete')
   //    ];
   //    return $functions;
   // }
   /* ---------------------------------------------- */
   /* Extra Table Nav */
   /* ---------------------------------------------- */
   protected function extra_tablenav($which)
   {
      echo '<div class="alignleft actions">';
      if ('top' === $which) {
         $output = $this->tableNavLCIDField();
         if (!empty($output)) {
            echo $output;
            submit_button(__('Filter'), '', 'filter_action', false, array('id' => 'post-query-submit'));
         }
      }
      echo '</div>' . PHP_EOL;
   }
}
